# -*- coding: utf-8 -*-
"""
Created on Mon Aug 24 17:42:10 2020

@author: omedeiro
"""

import time
from time import sleep

import numpy as np

import qnnpy.functions.functions as qf


class nTron:
    """Class for nTron measurement. This class handels the instrument
    configuration, measurement, plotting, logging, and saving.
    """

    def __init__(self, configuration_file):
        #######################################################################
        #       Load .yaml configuraiton file
        #######################################################################

        # qf is a package of base functions that can be used in any class.
        self.properties = qf.load_config(configuration_file)

        self.sample_name = self.properties["Save File"]["sample name"]
        self.device_name = self.properties["Save File"]["device name"]
        self.device_type = self.properties["Save File"]["device type"]

        self.isw = 0
        self.instrument_list = []

        self.R_srs = self.properties["iv_sweep"]["series_resistance"]
        self.R_srs_g = self.properties["iv_sweep"]["series_resistance_g"]

        # set up and store instruments
        self.inst = qf.Instruments(self.properties)

        # if there are any errors in setting up, compare older versions of
        # ntron on github with qf.Instruments

    def voltage2current(
        self, voltage: float, attenuation: float, resistance: float = 50
    ):
        voltage = voltage * 10 ** (attenuation / 20)
        current = voltage / resistance
        return current

    def current2voltage(
        self, current: float, attenuation: float, resistance: float = 50
    ):
        voltage = current * resistance
        voltage = voltage / 10 ** (attenuation / 20)
        return voltage


class IvSweep(nTron):
    """Class object for iv sweeps

    Configuration: iv_sweep:
    [start: initial bias current],
    [stop: final bias current],
    [steps: number of points],
    [sweep: number of itterations],
    [full_sweep: Include both positive and negative bias],
    [series_resistance: resistance at voltage source ie R_srs]

    """

    def run_sweep_fixed(self):
        """Runs IV sweep with config paramters
        This constructs #steps between start and 75% of final current.
        Then, #steps between 75% and 100% of final current.

        np.linspace()
        """
        # self.inst.source.reset()
        # self.inst.meter.reset()

        self.inst.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        steps = self.properties["iv_sweep"]["steps"]
        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop, steps)  # Coarse
        # Isource2 = np.linspace(stop*0.75, stop, steps) #Fine

        if full_sweep:
            Isource = np.concatenate([Isource1, Isource1[::-1]])
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = Isource1
        self.v_set = np.tile(Isource, sweep) * self.R_srs

        self.inst.source.set_output(True)
        sleep(1)
        voltage = []
        current = []
        self.i_set = 0
        # self.inst.temp1 = self.inst.temp.read_temp(self.inst.temp.channel)
        for n in self.v_set:
            self.inst.source.set_voltage(n)
            sleep(0.1)

            vread = self.inst.meter.read_voltage()  # Voltage

            iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def run_sweep_basic(self):
        """Runs IV sweep with config paramters


        np.linspace()
        """
        self.inst.source.reset()
        self.inst.meter.reset()

        self.inst.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        steps = self.properties["iv_sweep"]["steps"]
        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop, steps)

        if full_sweep:
            Isource = np.concatenate([Isource1, Isource1[::-1]])
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource, -Isource])
        self.v_set = np.tile(Isource, sweep) * self.R_srs

        self.inst.source.set_output(True)
        sleep(1)
        voltage = []
        current = []

        self.i_set = 0
        for n in self.v_set:
            self.inst.source.set_voltage(n)
            sleep(0.1)

            vread = self.inst.meter.read_voltage()  # Voltage

            iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def run_sweep_dynamic(self):
        """Runs IV sweep with config paramters
        This constructs [points_coarse] between start and [percent] of final current.
        Then, [points_fine] between [percent] and 100% of final current.

        uses np.linspace()
        """
        self.inst.source.reset()
        self.inst.meter.reset()

        self.inst.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        percent = self.properties["iv_sweep"]["percent"]
        num1 = self.properties["iv_sweep"]["points_coarse"]
        num2 = self.properties["iv_sweep"]["points_fine"]

        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop * percent, num1)  # Coarse
        Isource2 = np.linspace(stop * percent, stop, num2)  # Fine

        if full_sweep:
            Isource = np.concatenate(
                [Isource1, Isource2, Isource2[::-1], Isource1[::-1]]
            )
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource1, Isource2])
        self.v_set = np.tile(Isource, sweep) * self.R_srs

        self.inst.source.set_output(True)
        sleep(1)
        voltage = []
        current = []

        self.i_set = 0
        for n in self.v_set:
            self.inst.source.set_voltage(n)
            sleep(0.1)

            vread = self.inst.meter.read_voltage()  # Voltage

            iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def run_sweep_spacing(self):
        """Runs IV sweep with config paramters
        This constructs [points_coarse] between start and [percent] of final current.
        Then, [points_fine] between [percent] and 100% of final current.

        uses np.arange()
        """
        self.inst.source.reset()
        self.inst.meter.reset()

        self.inst.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        percent = self.properties["iv_sweep"]["percent"]
        spacing1 = self.properties["iv_sweep"]["spacing_coarse"]
        spacing2 = self.properties["iv_sweep"]["spacing_fine"]

        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.arange(start, stop * percent + spacing1, spacing1)  # Coarse
        Isource2 = np.arange(stop * percent, stop + spacing2, spacing2)  # Fine

        if full_sweep:
            Isource = np.concatenate(
                [Isource1, Isource2, Isource2[::-1], Isource1[::-1]]
            )
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource1, Isource2])
        self.v_set = np.tile(Isource, sweep) * self.R_srs

        self.inst.source.set_output(True)
        sleep(1)
        voltage = []
        current = []

        self.i_set = 0
        for n in self.v_set:
            self.inst.source.set_voltage(n)
            sleep(0.1)

            vread = self.inst.meter.read_voltage()  # Voltage

            iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def run_sweep_2400(self):
        self.inst.source.setup_2W_source_I_read_V()
        self.inst.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        steps = self.properties["iv_sweep"]["steps"]
        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop, steps)

        if full_sweep:
            Isource = np.concatenate([Isource1, Isource1[::-1]])
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource, -Isource])
        self.i_set = np.tile(Isource, sweep)

        self.inst.source.set_output(True)
        sleep(1)
        voltage = []
        current = []

        self.v_set = 0
        for n in self.i_set:
            self.inst.source.set_current(n)
            sleep(0.1)
            vread, iread = self.inst.source.read_voltage_and_current()

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    # def run_sweep_yoko(self):
    #         self.inst.source.setup_source_current()
    #         self.inst.source.set_output(False)

    #         start = self.properties['iv_sweep']['start']
    #         stop = self.properties['iv_sweep']['stop']
    #         steps = self.properties['iv_sweep']['steps']
    #         sweep = self.properties['iv_sweep']['sweep']
    #         # To select full (positive and negative) trace or half trace
    #         full_sweep = self.properties['iv_sweep']['full_sweep']
    #         Isource1 = np.linspace(start, stop, steps)

    #         if full_sweep == True:
    #             Isource = np.concatenate([Isource1, Isource1[::-1]])
    #             Isource = np.concatenate([Isource, -Isource])
    #         else:
    #             Isource = np.concatenate([Isource, -Isource])
    #         self.i_set = np.tile(Isource, sweep)

    #         self.inst.source.set_output(True)
    #         sleep(1)
    #         voltage = []
    #         current = []

    #         self.v_set=0
    #         for n in self.i_set:
    #             self.inst.source.set_current(n)
    #             sleep(0.1)
    #             vread, iread = self.inst.source.read_voltage_and_current()

    #             print('V=%.4f V, I=%.2f uA, R =%.2f' %(vread, iread*1e6, vread/iread))
    #             voltage.append(vread)
    #             current.append(iread)

    #         self.v_read = voltage
    #         self.i_read = current

    def isw_calc(self):
        """Calculates critical current."""
        """
        Critical current is defined as the current one step before a voltage
        reading greater than 5mV.
        self.isw = self.i_read[np.argmax(np.array(self.v_read) > 0.005)-1]
        print('%.4f uA' % (self.isw*1e6))
        """

        """ New method looks at differential and takes average of points that 
        meet condition. Prints mean() and std().` std should be < 1-2µA
        """
        # isws = abs(self.i_read[np.where(abs(np.diff(self.i_read)/max(np.diff(self.i_read))) > 0.5)])
        # self.isw = self.i_read[np.argmax(np.array(self.v_read) > 0.005)-1]
        # print('%.4f uA' % (self.isw*1e6))

        try:
            switches = np.asarray(
                np.where(abs(np.diff(self.i_read) / max(np.diff(self.i_read))) > 0.8),
                dtype=int,
            ).squeeze()
            isws = abs(np.asarray([self.i_read[i] for i in switches]))
            print(isws)
            self.isw = isws.mean()
            print(
                "Isw_avg = %.4f µA :--: Isw_std = %.4f µA"
                % (self.isw * 1e6, isws.std() * 1e6)
            )
        except Exception:
            print("Could not calculate Isw. Isw set to 0")
            self.isw = 0

    def plot(self):
        full_path = qf.save(self.properties, "iv_sweep")
        if self.properties["Save File"].get("port"):
            qf.plot(
                np.array(self.v_read),
                np.array(self.i_read) * 1e6,
                title=self.sample_name
                + " "
                + self.device_type
                + " "
                + self.device_name
                + " "
                + self.properties["Save File"]["port"],
                xlabel="Voltage (V)",
                ylabel="Current (uA)",
                path=full_path,
                show=True,
                close=True,
            )
        else:
            qf.plot(
                np.array(self.v_read),
                np.array(self.i_read) * 1e6,
                title=self.sample_name
                + " "
                + self.device_type
                + " "
                + self.device_name,
                xlabel="Voltage (V)",
                ylabel="Current (uA)",
                path=full_path,
                show=True,
                close=True,
            )

    def save(self):
        # Set up data dictionary

        data_dict = {
            "V_source": self.v_set,
            "I_source": self.i_set,
            "V_device": self.v_read,
            "I_device": self.i_read,
            "Isw": self.isw,
            "R_srs": self.R_srs,
            "temp": self.properties["Temperature"]["initial temp"],
        }

        self.full_path = qf.save(
            self.properties, "iv_sweep", data_dict, instrument_list=self.instrument_list
        )


class DoubleSweep(nTron):
    """Class object for iv sweeps

    Configuration: iv_sweep:
    [start: initial bias current],
    [stop: final bias current],
    [steps: number of points],
    [sweep: number of iterations],
    [full_sweep: Include both positive and negative bias],
    [series_resistance: resistance at voltage source ie R_srs]

    """

    def run_sweep(self):
        """Runs IV sweep with config paramters
        This constructs #steps between start and 75% of final current.
        Then, #steps between 75% and 100% of final current.


        """
        self.inst.source.reset()
        self.inst.meter.reset()

        self.inst.source.set_output(False)
        self.inst.source2.set_output(False)

        start = self.properties["double_sweep"]["start"]
        stop = self.properties["double_sweep"]["stop"]
        steps = self.properties["double_sweep"]["steps"]
        sweep = self.properties["double_sweep"]["sweep"]
        Ig_start = self.properties["double_sweep"]["Ig_start"]
        Ig_stop = self.properties["double_sweep"]["Ig_stop"]
        Ig_steps = self.properties["double_sweep"]["Ig_steps"]

        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["double_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop, steps)  # Coarse
        # Isource2 = np.linspace(stop*0.75, stop, steps) #Fine

        Isource_Ig = np.linspace(Ig_start, Ig_stop, Ig_steps)

        if full_sweep:
            Isource = np.concatenate([Isource1, Isource1[::-1]])
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = Isource1
        self.v_set = np.tile(Isource, sweep) * self.R_srs
        self.v_set_g = np.tile(Isource_Ig, sweep) * self.R_srs_g

        self.inst.source.set_output(True)
        self.inst.source2.set_output(True)
        sleep(1)
        self.v_list = []
        self.i_list = []

        self.ig_list = []
        for m in self.v_set_g:
            voltage = []
            current = []
            self.inst.source2.set_voltage(m)
            sleep(0.1)
            for n in self.v_set:
                self.inst.source.set_voltage(n)
                sleep(0.1)

                vread = self.inst.meter.read_voltage()  # Voltage

                iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

                print(
                    "Vd=%.4f V, Id=%.2f uA, Ig=%.2f uA, R =%.2f"
                    % (vread, iread * 1e6, m * 1e6 / self.R_srs_g, vread / iread)
                )
                voltage.append(vread)
                current.append(iread)

            # ADDED by Andrew...
            # Turn off both voltage sources to provide reset time
            self.inst.source.set_voltage(0.0)
            self.inst.source2.set_voltage(0.0)
            sleep(2)
            #######

            self.v_list.append(np.asarray(voltage, dtype=np.float32))
            self.i_list.append(np.asarray(current, dtype=np.float32))
            self.ig_list.append(m / self.R_srs_g)

    def plot(self):
        full_path = qf.save(self.properties, "double_sweep")
        qf.plot(
            self.v_list,
            self.i_list,
            title=self.sample_name + " " + self.device_type + " " + self.device_name,
            xlabel="Voltage (V)",
            ylabel="Current (A)",
            label=self.ig_list,
            path=full_path,
            show=True,
            close=True,
        )

    def save(self):
        # Set up data dictionary

        data_dict = {
            "V_source": self.v_set,
            "V_device": self.v_list,
            "I_device": self.i_list,
            "I_gate": self.ig_list,
            "R_srs": self.R_srs,
            "temp": self.properties["Temperature"]["initial temp"],
        }

        self.full_path = qf.save(
            self.properties,
            "double_sweep",
            data_dict,
            instrument_list=self.instrument_list,
        )


# class DoubleSweepScope(nTron):
#     """ use awg and scope to aquire Ic distributions.
#         Awg settings and scope settings are not currently programmed.

#     """
#     def run_sweep(self):
#         'runs the sweep'

#         self.inst.source.reset()
#         self.inst.source.set_output(False)
#         self.R_srs = self.properties['double_sweep_scope']['series_resistance_srs']
#         start = self.properties['double_sweep_scope']['start']
#         stop = self.properties['double_sweep_scope']['stop']
#         steps = self.properties['double_sweep_scope']['steps']
#         sweep = self.properties['double_sweep_scope']['sweep']

#         trace_signal = self.properties['double_sweep_scope']['trace_signal']
#         trace_trigger = self.properties['double_sweep_scope']['trace_trigger']
#         trace_hist = self.properties['double_sweep_scope']['trace_hist']

#         Isource = np.linspace(start, stop, steps)

#         self.v_set = np.tile(Isource, sweep) * self.R_srs


#         for i in self.v_set:
#             self.inst.source.set_voltage(i)
#             self.inst.source.set_output(True)
#             print('Voltage:%0.2f ' % i)
#             sleep(0.1)

#             self.inst.scope.set_trigger_mode(trigger_mode = 'Stop')
#             self.inst.scope.math_histogram_clear_sweeps()

#             self.data_dict = self.inst.scope.save_traces_multiple_sequence(
#                 channels = [trace_signal, trace_trigger],
#                 num_traces = 1,
#                 NumSegments = 1000)
#             hist = self.inst.scope.get_wf_data(trace_hist)
#             self.data_dict['hist'] = hist
#             self.data_dict['i_bias'] = i/self.R_srs

#             self.full_path = qf.save(self.properties, 'double_sweep_scope', self.data_dict,
#                                      instrument_list = self.instrument_list)
#             self.inst.scope.save_screenshot(file_name=self.full_path+'.png', white_background=False)

#         self.inst.scope.set_segments(2)


class ProbeStationDoubleSweep(nTron):
    """Class object for iv sweeps

    Configuration: iv_sweep:
    [start: initial bias current],
    [stop: final bias current],
    [steps: number of points],
    [sweep: number of iterations],
    [full_sweep: Include both positive and negative bias],
    [series_resistance: resistance at voltage source ie R_srs]

    """

    def run_sweep(self):
        """Runs IV sweep with config paramters
        This constructs #steps between start and 75% of final current.
        Then, #steps between 75% and 100% of final current.


        """
        self.inst.source.reset()
        self.inst.meter.reset()
        self.inst.meter2.setup_read_volt()
        self.inst.meter2.set_output(output=True)

        self.inst.source.set_output(False)
        self.inst.source2.set_output(False)

        start = self.properties["double_sweep"]["start"]
        stop = self.properties["double_sweep"]["stop"]
        steps = self.properties["double_sweep"]["steps"]
        sweep = self.properties["double_sweep"]["sweep"]
        Ig_start = self.properties["double_sweep"]["Ig_start"]
        Ig_stop = self.properties["double_sweep"]["Ig_stop"]
        Ig_steps = self.properties["double_sweep"]["Ig_steps"]

        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["double_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop, steps)  # Coarse
        # Isource2 = np.linspace(stop*0.75, stop, steps) #Fine

        Isource_Ig = np.linspace(Ig_start, Ig_stop, Ig_steps)

        if full_sweep:
            Isource = np.concatenate([Isource1, Isource1[::-1]])
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = Isource1
        self.v_set = np.tile(Isource, sweep) * self.R_srs
        self.v_set_g = np.tile(Isource_Ig, sweep) * self.R_srs_g

        self.inst.source.set_output(True)
        self.inst.source2.set_output(True)
        sleep(1)
        self.v_list = []
        self.i_list = []

        self.ig_list = []
        self.g_vlist = []
        self.g_ilist = []
        for m in self.v_set_g:
            voltage = []
            current = []
            v_g = []
            i_g = []
            self.inst.source2.set_voltage(m)
            sleep(0.1)
            for n in self.v_set:
                self.inst.source.set_voltage(n)
                sleep(0.1)

                vread = self.inst.meter.read_voltage()  # Voltage
                vgread = self.inst.meter2.read_voltage()

                iread = (n - vread) / self.R_srs  # (set voltage - read voltage)
                # igread = (m-vgread)/self.R_srs_g

                print(
                    "Vd=%.4f V, Id=%.2f uA, Ig=%.2f uA, R =%.2f"
                    % (vread, iread * 1e6, m * 1e6 / self.R_srs_g, vread / iread)
                )
                voltage.append(vread)
                current.append(iread)
                v_g.append(vgread)
                # i_g.append(igread)

            # ADDED by Andrew...
            # Turn off both voltage sources to provide reset time
            self.inst.source.set_voltage(0.0)
            self.inst.source2.set_voltage(0.0)
            sleep(2)
            #######

            self.v_list.append(np.asarray(voltage, dtype=np.float32))
            self.i_list.append(np.asarray(current, dtype=np.float32))
            self.g_vlist.append(np.asarray(v_g, dtype=np.float32))
            # self.g_ilist.append(np.asarray(i_g, dtype=np.float32))
            self.ig_list.append(m / self.R_srs_g)

    def plot(self):
        full_path = qf.save(self.properties, "double_sweep")
        qf.plot(
            self.v_list,
            self.i_list,
            title=self.sample_name + " " + self.device_type + " " + self.device_name,
            xlabel="Voltage (V)",
            ylabel="Current (A)",
            label=self.ig_list,
            path=full_path,
            show=True,
            close=True,
        )
        qf.plot(
            self.i_list,
            self.g_vlist,
            title=self.sample_name + " " + self.device_type + " " + self.device_name,
            xlabel="Voltage (V)",
            ylabel="Current (A)",
            label=self.ig_list,
            path=full_path,
            show=True,
            close=True,
        )

    def save(self):
        # Set up data dictionary

        data_dict = {
            "V_source": self.v_set,
            "V_device": self.v_list,
            "I_device": self.i_list,
            "I_gate": self.ig_list,
            "R_srs": self.R_srs,
            "temp": self.properties["Temperature"]["initial temp"],
        }

        self.full_path = qf.save(
            self.properties,
            "double_sweep",
            data_dict,
            instrument_list=self.instrument_list,
        )


class IvSweepScope(nTron):
    """
    iv_sweep_scope:
      awg_amp: 0.06
      atten: 0
      freq: 100
      trigger_v: 0.1
      trigger_channel: 'C4'
      channels: [C2,F1, F2]
      hist_channel: 'F3'
      num_segments: 500
    """

    def run_sweep(self, wf="RAMP"):
        awg_amp = self.properties["iv_sweep_scope"]["awg_amp"]
        atten = self.properties["iv_sweep_scope"]["atten"]
        freq = self.properties["iv_sweep_scope"]["freq"]

        trigger_v = self.properties["iv_sweep_scope"]["trigger_v"]
        trigger_channel = self.properties["iv_sweep_scope"]["trigger_channel"]
        channels = self.properties["iv_sweep_scope"]["channels"]
        num_segments = self.properties["iv_sweep_scope"]["num_segments"]

        if wf == "RAMP":
            self.inst.awg.set_waveform("RAMP", freq=freq, amplitude=awg_amp)
            self.inst.awg.write("FUNCtion:RAMP:SYMM 50")
        if wf == "PULSE":
            self.inst.awg.set_pulse(
                freq=freq, vlow=0.0, vhigh=awg_amp, width=6.6e-6, chan=1
            )

        temperature1 = self.inst.temp.read_temp()
        self.inst.scope.set_trigger(source=trigger_channel, volt_level=trigger_v)
        self.inst.scope.pyvisa.timeout = 10000
        self.inst.scope.clear_sweeps()
        data = self.inst.scope.get_multiple_trace_sequence(
            channels=channels, NumSegments=num_segments
        )

        key = self.properties.get("iv_sweep_scope")
        if key.get("hist_channel"):
            hist_channel = self.properties["iv_sweep_scope"]["hist_channel"]
            hist = self.inst.scope.get_wf_data(hist_channel)
            hist_dict = {hist_channel + "x": hist[0], hist_channel + "y": hist[1]}
            data.update(hist_dict)

        temperature2 = self.inst.temp.read_temp()

        self.inst.scope.set_sample_mode()
        self.inst.scope.set_trigger_mode()

        data.update(
            {
                "freq": freq,
                "awg_amp": awg_amp,
                "atten": atten,
                "temp1": temperature1,
                "temp2": temperature2,
            }
        )

        self.data_dict_iv_sweep_scope = data

        self.inst.scope.set_trigger_mode()

    def save(self):
        self.full_path = qf.save(
            self.properties,
            "iv_sweep_scope",
            self.data_dict_iv_sweep_scope,
            instrument_list=self.instrument_list,
        )
        self.inst.scope.save_screenshot(
            self.full_path + "screen_shot" + ".png", white_background=False
        )


class PulseTraceCurrentSweep(nTron):
    """UNFINISHED Sweep current and acquire trace. Configuration requires
    pulse_trace section: {start, stop, steps}"""

    def trace_data(self):
        start = self.properties["pulse_trace"]["start"]
        stop = self.properties["pulse_trace"]["stop"]
        steps = self.properties["pulse_trace"]["steps"]
        num_traces = 1
        currents = np.linspace(start, stop, steps)

        snspd_traces_x_list = []
        snspd_traces_y_list = []
        pd_traces_x_list = []
        pd_traces_y_list = []
        start_time = time.time()
        self.inst.source.set_output(True)
        for n, i in enumerate(currents):
            print(
                "   ---   Time elapsed for measurement %s of %s: %0.2f "
                "min    ---   " % (n, len(currents), (time.time() - start_time) / 60.0)
            )
            self.inst.source.set_voltage(i * self.R_srs)
            pd_traces_x = []  # Photodiode pulses
            pd_traces_y = []
            snspd_traces_x = []  # Device pulses
            snspd_traces_y = []
            self.inst.scope.clear_sweeps()
            for n in range(num_traces):
                x, y = self.inst.source.get_single_trace(channel="C2")
                snspd_traces_x.append(x)
                snspd_traces_y.append(y)
                x, y = self.inst.source.get_single_trace(channel="C3")
                pd_traces_x.append(x)
                pd_traces_y.append(y)

            snspd_traces_x_list.append(snspd_traces_x)
            snspd_traces_y_list.append(snspd_traces_y)
            pd_traces_x_list.append(pd_traces_x)
            pd_traces_y_list.append(pd_traces_y)

    def save(self):
        """write"""



# note: need to pip install LabJackPython to use
import u12


class LabJackU12(object):
    """Python class for LabJack U12, written by Emma Batson.
    https://labjack.com/products/u12
    https://labjack.com/support/software/examples/u12/python
    """

    def __init__(self, debug=False):
        self.device = u12.U12(debug=debug)
        self.read_digital(0)
        # note: may need to collect calibration data?

    def write_digital(self, channel, state):
        self.device.eDigitalOut(channel, state)

    def read_digital(self, channel):
        """
        note: this operation may reset the channel to 0 after readout
        """
        read_info = self.device.eDigitalIn(channel)
        return read_info["state"]

    def read_analog(self, channel):
        read_info = self.device.eAnalogIn(channel)
        return read_info["voltage"]

    def set_mux_channel(self, mux_channel, mapping, mux_names):
        """
        mux_channel: channel to switch mux to (base ten)
        mapping: mapping of LabJack digital outputs to
        control device input labels
        mux_names: list of mux channel names used in mapping, in order

        writes the number corresponding to the mux channel
        to LabJack digital outputs
        """
        mux_bin = bin(mux_channel)[2:]
        # prepend 0s for any unused control bits
        while len(mux_bin) < len(mux_names):
            mux_bin.insert(0, 0)
        # reverse endianness so it makes sense
        mux_bin_little = mux_bin[::-1]

        # enable mux
        enable_channel = mapping["EN"]
        self.write_digital(enable_channel, 0)

        # apply digital output according to mapping
        for mux_in, bit in zip(mux_names, mux_bin_little):
            digital_channel = mapping[mux_in]
            self.write_digital(digital_channel, int(bit))

        """mux_bin = bin(mux_channel)[2:]
        print(mux_bin)
        # reverse endianness (direction)
        mux_bin_little = mux_bin[::-1]
        print(mux_bin_little)
        for i in range(len(mux_bin_little)):
            self.write_digital(i, mux_bin_little[i])"""



import atexit
import logging
import os
import signal
import sys
import time

from ice_logging import read_ice_log

sys.path.append(r"Q:\qnnpy")

PATH = "S:/SC/InstrumentLogging/Cryogenics/Ice/ice-log/Results/"

# Setup logging
log_file = r"C:\Users\ICE\ice-logging\logfile.txt"
logging.basicConfig(
    filename=log_file, level=logging.INFO, format="%(asctime)s %(message)s"
)

logging.info("Script started.")


# Define the flag file path
flag_file = r"C:\Users\ICE\ice-logging\flagfile.txt"

# Create the flag file
try:
    with open(flag_file, "w") as f:
        f.write("Running")
    logging.info("Flag file created.")
except Exception as e:
    logging.error(f"Error creating flag file: {e}")


# Register the flag file for removal on exit
def remove_flag_file():
    if os.path.exists(flag_file):
        os.remove(flag_file)


atexit.register(remove_flag_file)


def signal_handler(sig, frame):
    logging.info("Signal received: %s. Cleaning up and exiting.", sig)
    remove_flag_file()
    sys.exit(0)


signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)


try:
    while True:
        read_ice_log(PATH)
        time.sleep(120)
        pass
except KeyboardInterrupt:
    logging.info("KeyboardInterrupt received. Exiting.")
finally:
    logging.info("Script ended.")



import pyvisa


class SPD3303(object):
    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # set response timeout (in milliseconds)

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        return self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    #### implement functions specific to this machine ####

    # turn on channel
    def set_channel_on(self, channel):
        # input: channel number as string or int
        string = "OUTPut CH{},ON"
        self.write(string.format(channel))

    # turn off channel
    def set_channel_off(self, channel):
        # input: channel number (str or int)
        string = "OUTPut CH{},OFF"
        self.write(string.format(channel))

    # set selected channel
    def set_selected_channel(self, channel):
        # input: channel number (str or int)
        string = "INSTrument CH{}"
        self.write(string.format(channel))

    # get selected channel
    def get_selected_channel(self):
        # return: currently selected channel (str)
        string = "INSTrument?"
        return self.query(string)

    # set current
    def set_current(self, channel, current):
        # inputs: channel number (str or int), current (str or float or int)
        string = "CH{}:CURRent {}"
        self.write(string.format(channel, current).strip())

    # get current
    def get_current(self, channel):
        # inputs: channel number (str or int)
        string = "CH{}:CURRent?"
        return self.query(string.format(channel).strip())

    # set voltage
    def set_voltage(self, channel, voltage):
        # inputs: channel number (str or int), voltage (str or float or int)
        string = "CH{}: VOLTage {}"
        self.write(string.format(channel, voltage).strip())

    # get voltage
    def get_voltage(self, channel):
        # inputs: channel number (str or int)
        string = "CH{}: VOLTage?"
        return self.query(string.format(channel).strip())

    # query for errors (returns 0 if no error)
    def query_error(self):
        string = "SYSTem: ERRor?"
        return self.query(string)



import pyvisa


class HP3748A(object):
    """Python class for HP 3478A Multimeter, written by Adam McCaughan"""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)
        # Anything else here that needs to happen on initialization

    # def reset(self):
    # self.write('*RST')
    # def read_volt(self):
    # self.query('VOLT %0.6e' %(vpp))
    # def set_output(self,message='OFF'):
    # self.write_simport('OP' + message[0:2])  # Only uses "OPOF" or "OPON": "OPOFF" does not work



# -*- coding: utf-8 -*-
"""
Created on Thu Feb 10 15:02:29 2022

@author: QNN_LabUser
"""

import visa


class RedPitaya(object):
    """

    Python Class for RedPitaya. Created by Owen Medeiros 2022.

    example visa_name 'TCPIP::18.25.31.186::5000::SOCKET'

    """

    def __init__(self, visa_name):
        rm = visa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name, read_termination="\r\n")
        self.pyvisa.timeout = 10000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        text = self.pyvisa.read()
        text = text.replace("ERR!", "")
        return text

    def read_raw(self):
        return self.pyvisa.read_raw()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def get_buffer_size(self):
        return int(self.query("ACQ:BUF:SIZE?"))

    def set_output(self, output=True, chan=1):
        if output:
            self.write("OUTPUT%s:STATE ON" % chan)
        else:
            self.write("OUTPUT%s:STATE OFF" % chan)

    def set_freq(self, freq, chan=1):
        print("SOUR%s:FREQ:FIX %s" % (chan, freq))
        self.write("SOUR%s:FREQ:FIX %s" % (chan, freq))

    def set_waveform(self, wf="SINE", chan=1):
        """{SINE, SQUARE, TRIANGLE, SAWU, SAWD, PWM, ARBITRARY, DC, DC_NEG}"""
        self.write("SOUR%s:FUNC %s" % (chan, wf))

    def set_amplitude(self, voltage, chan=1):
        self.write("SOUR%s:VOLT %s" % (chan, voltage))

    def send_waveform(self, string, chan=1):
        self.write("SOUR%s:TRAC:DATA:DATA " % (chan) + string)


# rm = visa.ResourceManager()
# IP = '18.25.31.186' ## your Red Pitaya's IP
# PORT = 5000
# RP = rm.open_resource('TCPIP::' + IP + '::' + str(PORT) + '::SOCKET', read_termination='\r\n')
# for i in range(1, 8):
#     RP.write('DIG:PIN LED' + str(i) + ',' + str(1))
#     time.sleep(0.1)
#     RP.write('DIG:PIN LED' + str(i) + ',' + str(0))
#     time.sleep(0.1)

# print(RP.query('*IDN?'))



# -*- coding: utf-8 -*-
"""
Created on Tue Jun 16 12:50:01 2020

@author: omedeiro & emmabat
"""

from time import sleep

import qnnpy.functions.functions as qf


class TcMeasurement:
    def __init__(self, configuration_file):
        #######################################################################
        #       Load .yaml configuraiton file
        #######################################################################

        # qf is a package of base functions that can be used in any class.
        self.properties = qf.load_config(configuration_file)

        # LabJack U12
        from qnnpy.instruments.labjack_u12 import LabJackU12

        self.mux = LabJackU12()
        self.mux_names = ["S0", "S1", "S2"]
        self.mapping = {"EN": 3, "S0": 2, "S1": 1, "S2": 0}

        # Temperature Controller
        if self.properties.get("Temperature"):
            ###################################################################
            if self.properties["Temperature"]["name"] == "Cryocon350":
                from qnnpy.instruments.cryocon350 import Cryocon350

                try:
                    self.temp = Cryocon350(self.properties["Temperature"]["port"])
                    self.channel = self.properties["Temperature"]["channel"]
                    self.properties["Temperature"]["initial temp"] = (
                        self.temp.read_temp(self.channel)
                    )
                    print("TEMPERATURE: connected")
                except Exception as e:
                    print("TEMPERATURE: failed to connect")
            ###################################################################
            elif self.properties["Temperature"]["name"] == "Cryocon34":
                from qnnpy.instruments.cryocon34 import Cryocon34

                try:
                    self.temp = Cryocon34(self.properties["Temperature"]["port"])
                    self.channel = self.properties["Temperature"]["channel"]
                    self.properties["Temperature"]["initial temp"] = (
                        self.temp.read_temp(self.channel)
                    )
                    print("TEMPERATURE: connected")
                except Exception:
                    print("TEMPERATURE: failed to connect")
            ###################################################################
            elif self.properties["Temperature"]["name"] == "ICE":
                try:
                    self.properties["Temperature"]["initial temp"] = qf.ice_get_temp()
                    print("TEMPERATURE: connected")
                except Exception:
                    print("TEMPERATURE: failed to connect")
            ###################################################################
            else:
                qf.lablog(
                    'Invalid Temperature Controller. TEMP name: "%s" is not configured'
                    % self.properties["Temperature"]["name"]
                )
                raise NameError(
                    'Invalid Temperature Controller. TEMP name: "%s" is not configured'
                    % self.properties["Temperature"]["name"]
                )

    def run_sweep(self, min_temp, num_channels, wait_time=1):
        """Sweep temperature and record voltage

        Configuration : tc_measurement:
        [start: initial temperature setpoint],
        [stop: final temperature setpoint],
        [iterations: number of start-stop-start sweeps]
        """
        temp = self.get_temp()

        # initialize storage variables
        self.temps = []
        self.all_voltages = []
        for channel in range(num_channels):
            self.all_voltages.append([])

        # while ICE cools to min_temp, read vals
        while temp > min_temp:
            # read temperature
            temp = self.get_temp()
            self.temps.append(temp)

            # read voltage from each analog channel
            for channel in range(num_channels):
                # LabJack functions execute in 20 ms or less
                # and set_mux_channel calls 4 LabJack functions
                self.mux.set_mux_channel(channel, self.mapping, self.mux_names)
                sleep(0.1)
                voltage = self.mux.read_analog(channel)
                sleep(0.05)
                self.all_voltages[channel - 1].append(voltage)
            sleep(wait_time)
        return self.temps, self.all_voltages

    def get_temp(self):
        if self.properties["Temperature"]["name"] == "ICE":
            return qf.ice_get_temp(select=1)
        else:
            return self.temp.read_temp(self.channel)

    def plot(self):
        full_path = qf.save(self.properties, "temperature sweep")
        qf.plot(
            self.temps,
            self.all_voltages,
            title=self.sample_name + " " + self.device_type,
            xlabel="Temperature [K]",
            ylabel="Voltage [V]",
            path=full_path,
            show=True,
            linestyle="-",
            close=True,
        )

    def save(self):
        data_dict = {
            "temps": self.temps,
            "voltages": self.all_voltages,
        }
        qf.save(
            self.properties,
            "temperature sweep",
            data_dict,
            instrument_list=self.instrument_list,
        )



import pyvisa


class Agilent8153A(object):
    """Python class for Agilent 8153A power meter, written by Adam McCaughan"""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def reset(self):
        self.write("*RST")

    def setup_basic(self, lambda_nm=1550, averaging_time=0.1):
        self.write("*RST")
        self.write("INIT1:CONT 1")
        self.set_averaging_time(
            averaging_time=averaging_time
        )  # Sets averaging time, 20ms < value < 3600s
        self.set_wavelength(lambda_nm=lambda_nm)
        # self.write('OUTP 1')

    def set_averaging_time(self, averaging_time=0.1):
        self.write(
            "SENS1:POW:ATIME %0.3e" % averaging_time
        )  # Sets averaging time, 20ms < value < 3600s

    def read_power(self):
        power = float(self.query("READ1:POW?"))  # Returns power in watts
        return power

    def set_wavelength(self, lambda_nm):
        self.write("SENS1:POW:WAVE %0.6e" % (lambda_nm * 1e-9))


# pm = Agilent8153A('GPIB0::22')



import visa
from ThorlabsPM100 import ThorlabsPM100


class ThorlabsPM100Meta(ThorlabsPM100):
    def __init__(self, USB_address, verbose=False):
        rm = visa.ResourceManager()
        inst = rm.open_resource(USB_address)
        self._verbose = verbose
        self._inst = inst

    def reset(self):
        self.status.questionable.preset()

    def set_wavelength(self, wavelength):
        self.sense.correction.wavelength = wavelength
        # wavelength in nm

    def read_value(self):
        # stable = False
        # while not stable:
        #     value = self.read
        #     sleep(0.1)
        #     value1 = self.read
        #     if (value1 - value)/value < 1e-2
        #         stable = True
        self.sense.average.count = 10
        return self.read



# -*- coding: utf-8 -*-
"""
Created on Tue Jun 16 12:50:01 2020

@author: omedeiro
"""

import qnnpy.functions.functions as qf


class TcMeasurement:
    def __init__(self, configuration_file):
        #######################################################################
        #       Load .yaml configuraiton file
        #######################################################################

        # qf is a package of base functions that can be used in any class.
        self.properties = qf.load_config(configuration_file)

        # Temperature Controller
        if self.properties.get("Temperature"):
            ###################################################################
            if self.properties["Temperature"]["name"] == "Cryocon350":
                from qnnpy.instruments.cryocon350 import Cryocon350

                try:
                    self.temp = Cryocon350(self.properties["Temperature"]["port"])
                    channel = self.properties["Temperature"]["channel"]
                    self.properties["Temperature"]["initial temp"] = (
                        self.temp.read_temp(channel)
                    )
                    print("TEMPERATURE: connected")
                except Exception:
                    print("TEMPERATURE: failed to connect")
            ###################################################################
            elif self.properties["Temperature"]["name"] == "Cryocon34":
                from qnnpy.instruments.cryocon34 import Cryocon34

                try:
                    self.temp = Cryocon34(self.properties["Temperature"]["port"])
                    channel = self.properties["Temperature"]["channel"]
                    self.properties["Temperature"]["initial temp"] = (
                        self.temp.read_temp(channel)
                    )
                    print("TEMPERATURE: connected")
                except Exception:
                    print("TEMPERATURE: failed to connect")
            ###################################################################
            elif self.properties["Temperature"]["name"] == "ICE":
                try:
                    self.properties["Temperature"]["initial temp"] = qf.ice_get_temp()
                    print("TEMPERATURE: connected")
                except Exception:
                    print("TEMPERATURE: failed to connect")
            ###################################################################
            else:
                qf.lablog(
                    'Invalid Temperature Controller. TEMP name: "%s" is not configured'
                    % self.properties["Temperature"]["name"]
                )
                raise NameError(
                    'Invalid Temperature Controller. TEMP name: "%s" is not configured'
                    % self.properties["Temperature"]["name"]
                )

    def run_sweep(self):
        """Sweep temperature and record voltage

        Configuration : tc_measurement:
        [start: initial temperature setpoint],
        [stop: final temperature setpoint],
        [itterations: number of start-stop-start sweeps]

        If start is left blank then current temperature will be used as initial
        setpoint.
        """

        if self.properties["Temperature"]["name"] == "ICE":
            qf.ice_get_temp(select=1)

    def plot(self):
        NotImplemented("Plotting function not implemented yet.")

    def save(self):
        NotImplemented("Saving function not implemented yet.")



import numpy as np
import pyvisa


class Agilent33250a(object):
    """Python class for Agilent 33250a 80MHz Frequency Generator, written by Adam McCaughan"""

    # http://rfmw.em.keysight.com/bihelpfiles/Trueform/webhelp/US/Default.htm?lc=eng&cc=US&id=2197433

    # error messages
    # https://rfmw.em.keysight.com/wireless/helpfiles/e4982a/product_information/error_messages/error_messages.htm

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def beep(self):
        self.pyvisa.write("SYST:BEEP")

    def reset(self):
        self.write("*RST")

    def clear_volatile(self):
        self.write(":DATA:VOLatile:CLEar")

    def delete(self, name):
        self.write("DATA:DEL %s" % name)

    def get_catalog(self):
        "List the names of all waveforms in volatile memory currently available for selection."
        return self.query("DATA:CATalog?")

    def get_catalog_int(self):
        "List the names of sequence or arb files in internal memory"
        return self.query("MMEM:CAT:DATA:ARB?")

    def get_error(self):
        # https://rfmw.em.keysight.com/wireless/helpfiles/e4982a/product_information/error_messages/error_messages.htm
        print(self.query("SYST:ERR?"))

    def get_phase(self, chan=1):
        return self.query("SOURCE%s:PHAS?" % chan)

    def get_amplitude(self, chan=1):
        return self.query("SOURCE%s:VOLT?" % (chan))

    def get_freq(self, chan=1):
        return self.query("SOURCE%s:FREQ?" % (chan))

    def set_sin(self, freq=1000, vpp=0.1, voffset=0, chan=1):
        # In a string, %0.6e converts a number to scientific notation like
        # print '%.6e' %(1234.56789) outputs '1.234568e+03'
        self.write(
            "SOURCE%s:APPL:SIN %0.6e HZ, %0.6e VPP, %0.6e V"
            % (chan, freq, vpp, voffset)
        )

    def set_freq(self, freq=1000, chan=1):
        self.write("SOURCE%s:FREQ %0.6e" % (chan, freq))

    def set_high(self, v=0.1, chan=1):
        self.write("SOURCE%s:VOLT:HIGH %0.6e" % (chan, v))

    def set_low(self, v=0, chan=1):
        self.write("SOURCE%s:VOLT:LOW %0.6e" % (chan, v))

    def set_voffset(self, voffset=0.0, chan=1):
        self.write("SOURCE%s:VOLT:OFFS %0.6e" % (chan, voffset))

    def set_vpp(self, vpp=0.1, chan=1):
        self.write("SOURCE%s:VOLT %0.6e" % (chan, vpp))

    def set_arb_freq(self, freq=1e3, chan=1):
        self.write("SOURCE%s:FUNC:ARB:FREQ %0.6e" % (chan, freq))

    def set_arb_sample_rate(self, srate=1e4, chan=1):
        self.write("SOURCE%s:FUNC:ARB:SRAT %0.6e" % (chan, srate))

    def set_arb_sync(self):
        self.write("FUNC:ARB:SYNC")

    def set_vhighlow(self, vlow=0.0, vhigh=1.0):
        if vhigh > vlow:
            self.set_vpp(vhigh - vlow)
            self.set_voffset((vhigh + vlow) / 2.0)
            self.set_polarity(inverted=False)
        elif vhigh < vlow:
            self.set_vpp(vlow - vhigh)
            self.set_voffset((vhigh + vlow) / 2.0)
            self.set_polarity(inverted=True)

    def set_output(self, output=False, chan=1):
        if output is True:
            self.write("OUTPUT%s ON" % chan)
        else:
            self.write("OUTPUT%s OFF" % chan)

    def set_load(self, high_z=False, chan=1):
        if high_z is True:
            self.write("OUTP%s:LOAD INF" % chan)
        else:
            self.write("OUTP%s:LOAD 50" % chan)

    def set_polarity(self, inverted=False):
        if inverted is True:
            self.write("OUTP:POL INV")
        else:
            self.write("OUTP:POL NORM")

    def set_trigger(self, external_trigger=False, delay=0.0):
        if external_trigger:
            self.write("TRIG:SOUR EXT")
        else:
            self.write("TRIG:SOUR IMM")
        self.write("TRIG:DEL %s" % (delay))  # Delay in seconds

    def set_phase(self, value, chan=1, unit="DEG"):
        self.write("SOURCE%s:PHAS %s %s" % (chan, value, unit))

    def set_symmetry(self, value, chan=1):
        self.write("SOURCE%s:FUNCtion:RAMP:SYMMetry %s" % (chan, value))

    def trigger_now(self):
        self.write("*TRG")

    def set_burst_mode(self, burst_enable=True, num_cycles=1, phase=0, period=10e-6):
        if burst_enable:
            self.write("BURS:STAT ON")  # Enables burst state
            self.write("BURS:NCYC %s" % (num_cycles))
            self.write("BURS:PHAS %s" % (phase))  # Phase in degrees
            self.write("BURS:INT:PER %s" % (period))

        else:
            self.write("BURS:STAT OFF")  # Disables burst state

    def set_waveform(self, name="SIN", freq=1000, amplitude=0.1, offset=0, chan=1):
        """
        APPLy
        :SINusoid [<frequency> [,<amplitude> [,<offset>] ]]
        :SQUare [<frequency> [,<amplitude> [,<offset>] ]]
        :RAMP [<frequency> [,<amplitude> [,<offset>] ]]
        :PULSe [<frequency> [,<amplitude> [,<offset>] ]]♠
        :NOISe [<frequency|DEF>1 [,<amplitude> [,<offset>] ]]
        :DC [<frequency|DEF>1 [,<amplitude>|DEF>1 [,<offset>] ]]
        :USER [<frequency> [,<amplitude> [,<offset>] ]]
        """
        self.write(
            "SOURCE%s:APPL:%s %0.6e, %0.6e, %0.6e"
            % (chan, name, freq, amplitude, offset)
        )

    def set_pulse(
        self, freq=1000, vlow=0.0, vhigh=1.0, width=100e-6, edge_time=2.9e-9, chan=1
    ):
        vpp = vhigh - vlow
        voffset = vpp / 2
        self.write(
            "SOURCE%s:APPL:PULS %0.6e HZ, %0.6e VPP, %0.6e V"
            % (chan, freq, vpp, voffset)
        )
        self.write("SOURCE%s:PULS:WIDT %0.6e" % (chan, width))

    def set_pulse_tran(self, edge_time=5e-9, chan=1):
        self.write("SOUR%1.0d:FUNC:PULS:TRAN %0.6e" % (chan, edge_time))

    def set_pulse_width(self, width=1e-6, chan=1):
        self.write("SOURCE%s:PULS:WIDT %0.6e" % (chan, width))

    def set_pulse_lead(self, leading_edge_time=5e-9, chan=1):
        self.write("SOUR%1.0d:FUNC:PULS:TRAN:LEAD %0.6e" % (chan, leading_edge_time))

    # def set_pulse_trail(self, trailing_edge_time=5e-9, chan=1):
    #     self.write('SOUR%1.0d:FUNC:PULS:TRAN:TRAIL %0.6e' % (chan,trailing_edge_time))

    # def set_arb_wf(self, t = [0.0, 1e-3], v = [0.0,1.0], name = 'ARB_PY', num_samples=2**9, chan=1):
    #     """ Input voltage values will be scaled to +/-1.0, you can then adjust the overall
    #     amplitude using the set_vpp function.  The 33250a does not allow the input of time for each
    #     point, so we instead use interpolation here to create waveform of 2^14 equally-spaced
    #     points, after which you can use set_freq to get the desired freq"""

    #     t = np.array(t);  v = np.array(v)

    #     v = v-min(v);  v = 2*v/max(v);  v = v-1
    #     # temp = self.timeout; self.timeout = 60
    #     t_interp = np.linspace(t[0],t[-1],num_samples) # Can be up to 2**14 long
    #     v_interp = np.interp(t_interp, t, v)

    #     data_strings = ['%0.3f' % x for x in v_interp]
    #     data_msg = ', '.join(data_strings)

    #     # self.set_sin(chan=chan) #cannot delete selected waveform
    #     self.delete(name) #for overwriting existing name

    #     self.write('DATA VOLATILE, ' + data_msg) # Form of "DATA VOLATILE, 1, .67, .33, 0, -.33", p200 user's guide
    #     name = name[0:8].upper()
    #     self.write('DATA:COPY %s, VOLATILE' % name)
    #     self.write('SOURCE%s:APPL:USER' % chan)  # Set output to ARB
    #     self.write('FUNC:USER %s' % name) # Select the waveform in the volatile memory
    #     # self.write('APPL:USER')
    #     # self.timeout = temp
    #     # self.write('FUNC USER') # Output the selected waveform

    def set_arb_wf(self, waveform, name="ARB_PY", num_samples=2**9, chan=1):
        t = np.array(range(len(waveform)))
        t_interp = np.linspace(t[0], t[-1], num_samples)
        waveform = np.interp(t_interp, t, waveform)

        self.write("*CLS")  # clear status
        self.write(
            f"SOURce{chan}:DATA:VOLatile:CLEar"
        )  # clears volatile waveform memory
        self.query(
            f"SOUR{chan}:DATA:VOL:FREE?"
        )  # returns number of points available (free) in volatile memory
        # self.get_error() # check for error
        self.write(
            "FORM:BORD SWAP"
        )  # least-significant byte (LSB) of each data point is first. Most computers use this.

        self.query(
            "DATA:CAT?"
        )  # returns the contents of volatile waveform memory, including arbitrary waveforms and sequences

        waveform = waveform.astype("float32")  # set single precision
        mw = np.max(np.abs(waveform))  # find max for normalization
        # waveform = waveform/mw # normalize between +/-1
        arbBytes = str(len(waveform) * 4)  # number of bytes to send
        header = f"SOURce{chan}:DATA:ARBitrary {name}, #{len(arbBytes)}{arbBytes}"

        header = bytes(header, "ascii")
        binblockBytes = waveform.view(
            np.uint8
        )  # waveform as unsigned 8bit interger array
        data_msg = "".join([chr(x) for x in binblockBytes])
        data_msg = data_msg.encode("latin-1")
        message = header + data_msg
        self.pyvisa.write_raw(message)
        self.write("*WAI")

        self.write(f"SOURce{chan}:FUNCtion:ARBitrary {name}")
        self.write(f'MMEM:STOR:DATA{chan} "INT:\\{name}.arb"')
        self.write(f"SOUR{chan}:FUNC ARB")
        self.set_vpp(mw, chan)

    def set_arb_vpeak(self, v, chan):
        self.write(f"SOURce{chan}FUNC:ARBitrary:PTPeak {v}")

    def set_sequence(self, name, chan):
        self.write("*CLS")  # clear status
        self.write(
            f"SOURce{chan}:DATA:VOLatile:CLEar"
        )  # clears volatile waveform memory

        write_start = '"INT:\WRITE0.ARB",0,once,highAtStartGoLow,50'
        write1 = '"INT:\WRITE1.ARB",0,once,maintain,50'
        write0 = '"INT:\WRITE0.ARB",0,once,maintain,50'
        sequence = f'"{name}",' + ",".join(
            [write_start, write0, write0, write1, write1, write0, write1, write1]
        )
        n = str(len(sequence))
        header = f"SOUR{chan}:DATA:SEQ #{len(n)}{n}"
        message = header + sequence

        self.write(f'MMEM:LOAD:DATA{chan} "INT:\WRITE0.arb"')
        self.write(f'MMEM:LOAD:DATA{chan} "INT:\WRITE1.arb"')

        self.pyvisa.write_raw(message)
        self.write(f"SOURce{chan}:FUNCtion:ARBitrary {name}")
        # self.get_error()

    def setup_heartbeat_wf(self):
        heartbeat_t = [0.0, 4.0 / 8, 5.0 / 8, 6.0 / 8, 7.0 / 8, 8.0 / 8]
        heartbeat_v = [0.0, 0.0, 1.0, 0.0, -1.0, 0.0]
        self.set_arb_wf(t=heartbeat_t, v=heartbeat_v, name="HEARTBEA")

    def select_arb_wf(self, name="HEARTBEA"):
        name = name[0:8].upper()
        self.write("APPL:USER")  # Set output to ARB
        self.write("FUNC:USER %s" % name)
        self.write("APPL:USER")  # Set output to ARB

    def setup_pulse_train(
        self, number_of_pulses=6, t_delay=4, t_on=1, t_off=0.2, t_edge=0.01
    ):
        t = [0, t_delay]
        v = [0, 0]
        for i in range(number_of_pulses):
            t.append(round(t[-1] + t_edge, 2))
            t.append(round(t[-1] + t_on, 2))
            t.append(round(t[-1] + t_edge, 2))
            t.append(round(t[-1] + t_off, 2))

            v.extend([1, 1, 0, 0])
        self.get_catalog()
        self.set_arb_wf(t, v, name="PULSE_TR")

    def sync_phase(self, chan):
        # self.write('SOURCE%s:FUNC:ARBitrary:SYNChronize' % chan)
        self.write("SOURCE%s:PHASe:SYNChronize" % chan)



import pyvisa


class SIM928(object):
    """Python class for SRS SIM928 Isolated Voltage Source inside a SIM900"""

    def __init__(self, visa_name, sim900port):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands
        self.sim900port = sim900port
        # Anything else here that needs to happen on initialization

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def write_simport(self, message):
        write_str = "SNDT " + str(self.sim900port) + ',"' + message + '"'
        # print write_str
        self.write(write_str)  # Format of 'SNDT 4,\"GAIN 10\"'

    def ask_simport(self, message):
        write_str = "SNDT " + str(self.sim900port) + ',"' + message + '"'
        return self.query(write_str)  # Format of 'SNDT 4,\"GAIN 10\"'

    def reset(self):
        self.write_simport("*RST")

    def set_voltage(self, voltage=0.0):
        # In a string, %0.4e converts a number to scientific notation
        self.write_simport("VOLT %0.4e" % (voltage))

    def set_output(self, output=False):
        if output:
            self.write_simport("OPON")
        else:
            self.write_simport(
                "OPOF"
            )  # Only uses "OPOF" or "OPON": "OPOFF" does not work



# -*- coding: utf-8 -*-
"""
Created on Mon Jan 23 10:38:22 2023

@author: omedeiro
"""

import sys
from time import sleep

sys.path.append(r"Q:\qnnpy")
import qnnpy.functions.functions as qf
from qnnpy.instruments.lakeshore336 import Lakeshore336

while True:
    ls1 = Lakeshore336("GPIB0::12::INSTR")

    ls2 = Lakeshore336("GPIB0::13::INSTR")

    T_sample_stage = ls1.read_temp("A")
    T_4k_stage = ls1.read_temp("B")
    T_sample_holder = ls1.read_temp("C")
    # ls1.read_temp('D')

    T_rad_shield = ls2.read_temp("A")
    T_second_shield = ls2.read_temp("B")

    qf.log_data_to_database(
        table_name="probe_station_logging",
        sample_stage=T_sample_stage,
        fourK_stage=T_4k_stage,
        sample_holder=T_sample_holder,
        radiation_shield=T_rad_shield,
        second_shield=T_second_shield,
    )
    sleep(10)



# -*- coding: utf-8 -*-
"""
instrument class for HP 83711B signal generator
Created on Feb 26, 2020
@author: dizhu
"""

import pyvisa


class HP83711B(object):
    """Python class for HP 83711B, written by Di Zhu."""

    def __init__(self, visa_name, timeout=15000):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = timeout  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def idn(self):
        string = "*IDN?"
        return self.query(string)

    def reset(self):
        self.write("*RST")
        self.write(":UNIT:POW DBM")

    def get_freq(self):
        string = "FREQ?"
        return float(self.query(string).strip())

    def set_freq(self, freq=3.0):
        """
        this function sets frequency, in unit of GHz
        """
        string = "FREQ {}".format(freq * 1e9)
        self.write(string)

    def set_power(self, power=-30.0):
        """
        set power, in dBm, in .1 resolution
        """
        string = "POW {:.1f}DBM".format(power)
        self.write(string)

    def get_power(self):
        return float(self.query("POW?").strip())

    def get_output(self):
        state = self.query("OUTP?").strip()
        if state == "+1":
            return True
        else:
            return False

    def set_output(self, state=False):
        if state:
            string = ":OUTP ON"
        else:
            string = ":OUTP OFF"
        self.write(string)



import pyvisa


class Cryocon350(object):
    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def read_temp(self, channel="A"):
        self.write("KRDG? " + channel + "[term]")  # In form of ":INPUT? A:TEMP", A-D
        t = self.read()
        temperature = float(t[1:8])
        return temperature

        # see manual for other commands



from time import sleep

import numpy as np
import pyvisa


class N9020A(object):
    """Python class for Agilent (now Keyseight) N9020A signal analyzer, written by Di Zhu, 2020."""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.rsc = rm.open_resource(visa_name, read_termination="\n")
        self.rsc.timeout = 5000  # Set response timeout (in milliseconds)
        # self.rsc.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.rsc.read()

    def write(self, string):
        self.rsc.write(string)

    def query(self, string):
        return self.rsc.query(string)

    def idn(self):
        self.query("*IDN?")

    def reset(self):
        self.write("*RST")

    def get_freq_start(self):
        string = ":SENSe:FREQuency:STARt?"
        return float(self.query(string).strip())

    def set_freq_start(self, freq):
        string = ":SENSe:FREQuency:STARt {}".format(freq)
        self.write(string)

    def get_freq_stop(self):
        string = ":SENSe:FREQuency:STOP?"
        return float(self.query(string).strip())

    def set_freq_stop(self, freq):
        string = ":SENSe:FREQuency:STOP {}".format(freq)
        self.write(string)

    def set_freq_center(self, freq):
        string = ":SENSe:FREQuency:CENTer {}".format(freq)
        self.write(string)

    def get_freq_center(self):
        string = ":SENSe:FREQuency:CENTer?"
        return float(self.query(string).strip())

    def set_freq_span(self, freq):
        string = ":SENSe:FREQuency:SPAN {}".format(freq)
        self.write(string)

    def get_freq_span(self):
        string = ":SENSe:FREQuency:SPAN?"
        return float(self.query(string).strip())

    def get_avg_count(self):
        string = ":SENSe:AVERage:COUNt?"
        return int(self.query(string).strip())

    def set_avg_count(self, count):
        string = ":SENSe:AVERage:COUNt {}".format(count)
        self.write(string)

    def clear_avg(self):
        self.write(":SENSe:AVERage:CLEar")

    def get_bw(self):
        return float(self.query(":SENSe:BANDwidth:RESolution?").strip())

    def set_bw(self, bw):
        self.write(":SENSe:BANDwidth:RESolution {}".format(bw))

    def set_bw_auto(self, auto="True"):
        if auto:
            self.write(":SENSe:BANDwidth:AUTO ON")
        else:
            self.write(":SENSe:BANDwidth:AUTO OFF")

    def get_bw_auto(self):
        flag = self.query("BWID:AUTO?").strip()
        if flag == "0":
            return False
        if flag == "1":
            return True

    def get_data(self):
        data = self.query("FETCh:SANalyzer?").strip()
        data = data.split(",")
        f = data[0::2]
        f = [float(i) for i in f]
        p = data[1::2]
        p = [float(i) for i in p]
        return f, p

    def single_scan(self):
        self.write(":INIT:CONT OFF")
        self.write(":INITiate:RESTart")
        sleep(self.get_sweep_time())
        f, p = self.get_data()
        return f, p

    def get_sweep_time(self):
        return float(self.query(":SENS:SWE:TIME?").strip())

    def quick_scan(self, freq_center=2e9, freq_span=100e6, bw=100e3, avg=1):
        self.set_freq_center(freq_center)
        self.set_freq_span(freq_span)
        self.set_bw(bw)
        # self.set_avg_count(avg)
        f, p = self.averaged_scan(avg)
        return f, p

    def averaged_scan(self, avg=10):
        for i in range(avg):
            f, p = self.single_scan()
            if i == 0:
                p_total = np.array(p)
            else:
                p_total = p_total + np.array(p)
        return f, p_total / avg



# -*- coding: utf-8 -*-
"""
instrument class for YOKOGAWA AQ6370 OSA
Created on Fri Dec 27 2019
@author: dizhu
"""

from time import sleep

import numpy as np
import pyvisa


class AQ6370(object):
    """Python class for YOKOGAWA AQ6370 OSA, written by Di Zhu."""

    def __init__(self, visa_name, timeout=15000):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = timeout  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def idn(self):
        return self.query("*IDN?").strip()

    def opc(self):
        "ask if operation is completed"
        msg = self.query("*OPC?").strip()
        if msg == "1":
            return True
        else:
            return False

    def reset(self):
        self.write("*RST")

    def wait(self):
        self.write("*WAI")

    def repeat_sweep(self):
        self.write(":INITIATE:SMOD REPEAT")
        self.write("INIT")

    def abort(self):
        self.write("ABORt")

    def get_sweep_mode(self):
        """get the sweep mode
        1 = SINGle; 2 = REPeat; 3 = AUTO; 4 = SEGMent
        """
        msg = self.query(":INITiate:SMODe?").strip()
        if msg == "1":
            return "SINGLE"
        if msg == "2":
            return "REPEAT"
        if msg == "3":
            return "AUTO"
        if msg == "4":
            return "SEGMENT"

    def set_average(self, num_avg=1):
        """set the number of averagings"""
        self.write(":SENSE:AVERAGE:COUNT {:d}".format(int(num_avg)))
        pass

    def get_average(self):
        """get the number of averagings"""
        return int(self.query(":SENSE:AVERAGE:COUNT?").strip())

    def set_resolution(self, resolution=0.02e-9):
        """set resolution in nm, choices
        .02 nm, .1 nm, .2 nm, .5 nm, 1.0 nm, 2.0 nm"""
        if resolution not in [0.02e-9, 0.1e-9, 0.2e-9, 0.5e-9, 1.0e-9, 2, 0e-9]:
            print("ERROR: input not valid")
            return
        self.write(":SENSE:BANDWIDTH:RESOLUTION {}".format(resolution))

    def set_wl_center(self, wl=1550e-9):
        self.write(":SENSE:WAVELENGTH:CENTER {}".format(wl))

    def get_wl_center(self):
        """center wavelength in m"""
        return float(self.query(":SENSE:WAVELENGTH:CENTER?").strip())

    def set_wl_span(self, span=5e-9):
        self.write(":SENSE:WAVELENGTH:SPAN {}".format(span))

    def get_wl_span(self):
        return float(self.query(":SENSE:WAVELENGTH:SPAN?").strip())

    def set_wl_start(self, start=1550e-9):
        self.write(":SENSE:WAVELENGTH:START {}".format(start))

    def get_wl_start(self):
        return float(self.query(":SENSE:WAVELENGTH:START?").strip())

    def set_wl_stop(self, start=1550e-9):
        self.write(":SENSE:WAVELENGTH:STOP {}".format(start))

    def get_wl_stop(self):
        return float(self.query(":SENSE:WAVELENGTH:STOP?").strip())

    def get_trace(self):
        return self.query(":TRACE:ACTIVE?").strip()

    def set_trace(self, trace="A"):
        self.write(":TRACE:ACTIVE TR{}".format(trace))

    def single_sweep(self):
        self.write(":INITIATE:SMODE SINGLE")
        self.write("INIT")
        # self.write('*WAI')
        wl = self.query(":TRACE:X? {}".format(self.get_trace())).strip().split(",")
        power = self.query(":TRACE:Y? {}".format(self.get_trace())).strip().split(",")
        wl = [float(w) for w in wl]
        power = [float(p) for p in power]
        return np.array(wl), np.array(power)

    def quick_setup(
        self, wl_center=1550e-9, wl_span=2e-9, resolution=0.02e-9, average=1
    ):
        self.set_wl_center(wl_center)
        self.set_wl_span(wl_span)
        self.set_resolution(resolution)
        self.set_average(average)
        sleep(0.5)

    def quick_scan(self, wl_center=1550e-9, wl_span=2e-9):
        self.set_wl_center(wl_center)
        self.set_wl_span(wl_span)
        wl, power = self.single_sweep()
        return wl, power



# Ic measurement code
# Run add_path.py first
import sys

sys.path.append(r"Q:\qnnpy\qnnpy")

import time

import numpy as np
from matplotlib import pyplot as plt
from scipy.optimize import fmin


def setup_ic_measurement(
    lecroy,
    freq_gen,
    vpp=1,
    repetition_hz=100,
    trigger_level=0.1,
    trigger_slope="Positive",
    coupling_ch1="DC1M",
    coupling_ch2="DC1M",
):
    # Setup frequency generator to have heartbeat shape for Ic sweeping --'\,---
    freq_gen.set_load(high_z=True)
    # freq_gen.select_arb_wf(name = 'HEARTBEA')
    freq_gen.write("APPL:SIN 1000 HZ, 1 VPP, 0 V")

    # Change repetition rate and voltage amplitude
    freq_gen.set_freq(repetition_hz)
    freq_gen.set_vpp(vpp)
    freq_gen.set_output(True)

    # Setup LeCroy scope:
    lecroy.set_coupling(channel="C1", coupling=coupling_ch1)
    lecroy.set_coupling(channel="C2", coupling=coupling_ch2)
    lecroy.set_bandwidth(channel="C1", bandwidth="20MHz")
    lecroy.set_bandwidth(channel="C2", bandwidth="20MHz")
    lecroy.label_channel(channel="C1", label="AWG input waveform")
    lecroy.label_channel(channel="C2", label="Device output")
    lecroy.label_channel(channel="F1", label="Per-sweep Ic values")
    lecroy.label_channel(channel="F2", label="Ic values histogram")
    lecroy.set_vertical_scale(channel="C1", volts_per_div=500e-3, volt_offset=0)
    lecroy.set_vertical_scale(channel="C2", volts_per_div=100e-3, volt_offset=0)
    lecroy.set_horizontal_scale(
        time_per_div=(1.0 / repetition_hz) / 10.0, time_offset=0
    )
    lecroy.set_trigger(source="C2", volt_level=trigger_level, slope=trigger_slope)
    lecroy.set_trigger_mode(trigger_mode="Normal")
    lecroy.set_parameter(
        parameter="P1", param_engine="Mean", source1="C1", source2=None
    )
    lecroy.setup_math_trend(math_channel="F1", source="P1", num_values=10e3)
    lecroy.setup_math_histogram(math_channel="F2", source="P1", num_values=300)
    lecroy.set_parameter(
        parameter="P2", param_engine="LevelAtX", source1="C1", source2=None
    )
    lecroy.set_parameter(
        parameter="P3", param_engine="HistogramMedian", source1="F2", source2=None
    )
    lecroy.set_parameter(
        parameter="P4", param_engine="HistogramSdev", source1="F2", source2=None
    )
    lecroy.set_parameter(
        parameter="P5",
        param_engine="FullWidthAtHalfMaximum",
        source1="F2",
        source2=None,
    )
    lecroy.set_display_gridmode(gridmode="Single")


def run_ic_sweeps(lecroy, num_sweeps):
    lecroy.clear_sweeps()
    time.sleep(0.1)
    while lecroy.get_num_sweeps(channel="F1") < num_sweeps + 1:
        time.sleep(0.1)
    x, ic_values = lecroy.get_wf_data(channel="F1")
    while len(ic_values) < num_sweeps:
        x, ic_values = lecroy.get_wf_data(channel="F1")
        time.sleep(0.05)
    return ic_values[:num_sweeps]  # will occasionally return 1-2 more than num_sweeps


def gumbel_pdf(theta, x):
    a = theta[0]
    b = theta[1]
    return 1.0 / b * np.exp((x - a) / b - np.exp((x - a) / b))


def gumbel_dist_error_fun(theta, data, min_prob=1e-2):
    pdf_fun = gumbel_pdf
    prob = pdf_fun(theta=theta, x=data)
    prob[prob < min_prob] = (
        min_prob  # Otherwise any datapoint with probability zero returns log(P(x)=0) = -Inf
    )
    log_likelihood = np.sum(np.log(prob))
    return -log_likelihood


def analyze_ic_values(ic_values):
    theta0 = [np.median(ic_values), np.std(ic_values) * 10]
    thetaopt = fmin(gumbel_dist_error_fun, theta0, [ic_values])
    ic = thetaopt[0]
    delta_ic = thetaopt[1]
    return ic, delta_ic


def calc_ramp_rate(vpp, R, repetition_hz, wf="HEARTBEAT"):
    if wf.upper() == "HEARTBEAT":
        T = 1.0 / repetition_hz
        T_ramp = T / 8.0
        I_max = (vpp / 2.0) / R
        return I_max / T_ramp
    elif wf.upper() == "SINE":
        return (vpp / 2.0) / R * (2 * np.pi * repetition_hz)
    else:
        print('Incorrect waveform (must be "sine" or "heartbeat")')
        return 0


def data_list_to_histogram_list(
    data_list, num_bins=100, range_min=None, range_max=None, plotme=False
):
    hist_list = []
    for data in data_list:
        if range_min is None:
            range_min = np.min(data_list)
        if range_max is None:
            range_max = np.max(data_list)
        bin_vals, bin_edges = np.histogram(
            data, bins=num_bins, range=(range_min, range_max)
        )
        bin_centers = bin_edges[:-1] + (bin_edges[1] - bin_edges[0]) / 2.0
        hist_list.append(bin_vals)
    return hist_list, bin_centers


def quick_ic_test(lecroy, num_sweeps=1000, R=50):
    ic_data = run_ic_sweeps(lecroy, num_sweeps=num_sweeps) / R
    print("Median Ic = %0.2f uA / Std. dev Ic = %0.2f uA") % (
        np.median(ic_data * 1e6),
        np.std(ic_data * 1e6),
    )


def quick_retrap_test(lecroy, num_sweeps=1000, trigger_level=0.1, R=50):
    ic_data = run_ic_sweeps(lecroy, num_sweeps=num_sweeps) / R
    print("Median Iretrap = %0.2f uA / Std. dev Iretrap = %0.2f uA") % (
        np.median(ic_data * 1e6),
        np.std(ic_data * 1e6),
    )
    lecroy.set_trigger(source="C2", volt_level=trigger_level, slope="Positive")


def fit_ic_histogram(ic_values, bins=100, binrange=None, plot=False):
    ic_hist, bin_edges = np.histogram(a=ic_values, bins=bins)
    ic_hist_x = bin_edges[:-1] + (bin_edges[1] - bin_edges[0]) / 2.0

    ic, delta_ic = analyze_ic_values(ic_values)
    ic_hist_fit_x = np.linspace(min(ic_values), max(ic_values), bins)
    ic_hist_fit = gumbel_pdf([ic, delta_ic], ic_hist_fit_x)
    ic_hist_fit = ic_hist_fit / float(np.max(ic_hist_fit))
    ic_hist = ic_hist / float(np.max(ic_hist))
    if plot:
        plt.bar(ic_hist_x, ic_hist, width=(ic_hist_x[1] - ic_hist_x[0]))
        plt.plot(ic_hist_x, ic_hist_fit, "r")
        plt.show()
    return ic_hist_x, ic_hist, ic_hist_fit



import numpy as np
import pyvisa


class ThorlabsPM101A(object):
    """Python class for Thorlabs PM101A, written by Di Zhu.
    PM101A shares most of the SCIP commands as PM100D.
    One difference is the way it sets unit. In PM100D, it's POW:DC:UNIT, but PM101A uses SENS:POW:DC:UNIT
    same applies to auto range.
    """

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name, read_termination="\n")
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def reset(self):
        self.write("*RST")
        self.set_wavelength(1550.0)
        self.set_auto_range()
        print("wavelength: 1550 nm, range: auto")

    def set_auto_range(self, state=1):
        self.write("SENS:CURR:DC:RANG:AUTO{}".format(state))

    def get_auto_range(self):
        return self.query("SENS:CURR:DC:RANG:AUTO?")

    def set_wavelength(self, wl=1550):
        self.write("sense:corr:wav {}".format(wl))

    def get_wavelength(self):
        return float(self.query("sense:corr:wav?").strip())

    def get_power(self, unit="W", repetition=10):
        power = []
        # unit takes W or dBm
        if unit != "W" and unit != "dBm":
            print("invalid unit")
            return 0
        else:
            self.write("sens:power:dc:unit " + unit)
            for i in range(repetition):
                power.append(float(self.query("measure:power?").strip()))

            return np.mean(power)



from enum import Enum, auto

import pyvisa


class MeasType(Enum):
    def __str__(self):
        name = str(self.name)
        if name.isupper():
            return name
        elif "DC" or "AC" in name:
            return name[:-2].upper() + ":" + name[-2:].upper()
        else:
            return name[:-4].upper() + ":" + name[-4:].upper()

    VoltDC = auto()  # VOLT:DC
    CurrDC = auto()  # CURR:DC
    VoltAC = auto()  # VOLT:AC
    CurrAC = auto()  # CURR:AC
    RES = auto()  # 2-pt res
    FRES = auto()  # 4-pt res
    TEMP = auto()  # temp
    FREQ = auto()  # freq
    VoltFreq = auto()  # VOLT:FREQ
    CurrFreq = auto()  # CURR:FREQ


class Keithley2001(object):
    """Python class for Keithley 2001 Multimeter, written by Adam McCaughan"""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

        # Anything else here that needs to happen on initialization

    def reset(self):
        self.write("*RST")

    def local_key(self):
        self.write("SYST:KEY 7")

    def read_voltage(self):
        self.write(":CONF:VOLT:DC")
        read_str = self.query(":READ?")
        # Returns something like '+1.99919507E-01VDC,+6283.313SECS,+60584RDNG#'
        E_location = read_str.find("E")  # Finds location of E in read value
        voltage_str = read_str[0 : read_str.find("E") + 4]
        return float(voltage_str)  # Return just the first number (voltage)

    def read_value(self, value=MeasType.VoltDC):
        self.write(":CONF:" + str(value))
        read_str = self.query(":READ?")
        # Returns something like '+1.99919507E-01VDC,+6283.313SECS,+60584RDNG#'
        E_location = read_str.find("E")  # Finds location of E in read value
        value_str = read_str[0 : read_str.find("E") + 4]
        return float(value_str)  # Return just the first number (voltage)



# -*- coding: utf-8 -*-
"""
instrument class for HMC0T2240 singal gnerator
Created on Fri Dec 27 12:16:08 2019
@author: dizhu
"""

import pyvisa


class TMC_T2240(object):
    """Python class for HMC0T2240 singal gnerator, written by Di Zhu."""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def idn(self):
        return self.query("*IDN?").strip()

    def reset(self):
        self.write("*CLS;*SRE 0; *ESE 0; STATus:PRESet")

    def set_output(self, state=False):
        """turn on/off the output"""
        if not state:
            self.write("OUTP OFF")
        else:
            self.write("OUTP ON")

    def get_output(self):
        """get the status of the output"""
        result = self.query("OUTP?").strip()
        if result == "0":
            return "OFF"
        else:
            return "True"

    def set_freq(self, freq):
        """set frequency in GHz"""
        self.write("freq {}".format(freq * 1e9))

    def get_freq(self):
        """get frequency in GHz"""
        return float(self.query("freq?").strip()) / 1e9

    def set_power(self, power=-60, max_power=-12):
        """set power in dBm"""
        if power < max_power:
            self.write("POW {}".format(power))
        else:
            print(
                "Warning: set power exceeded preset limit, double confirm amplifier specs and explicitly adjust 'max_power'"
            )

    def get_power(self):
        """get power in dBm"""
        return float(self.query("POW?").strip())



import pyvisa


class StanfordSR830(object):
    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        self.pyvisa.write("OUTX 1")
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def set_ref_freq(self, freq):
        self.pyvisa.write("FREQ %0.6e" % freq)

    def set_ref_amp(self, amp):
        self.pyvisa.write("SLVL %0.6e" % amp)

    # def IV_curve(self, freq, I_max, R_srs, npoints):
    #     self.pyvisa.write('FMOD 1')
    #     self.set_ref_freq(freq)

    def set_ref(self, ref):
        if ref == "INT":
            self.pyvisa.write("FMOD 1")
        if ref == "EXT":
            self.pyvisa.write("FMOD 0")

    def set_input_mode(self, mode):
        "A (0), A-B (1) , I (1 MΩ) (2) or I (100 MΩ) (3)"
        if mode == "A":
            self.pyvisa.write("ISRC 0")
        if mode == "A-B":
            self.pyvisa.write("ISRC 1")
        if mode == "I (1 MΩ)":
            self.pyvisa.write("ISRC 2")
        if mode == "I (100 MΩ)":
            self.pyvisa.write("ISRC 03")

    def set_sensitivity(self, sens):
        "Set (Query) the Sensitivity to 2 nV (0) through 1 V (26) rms full scale"
        self.pyvisa.write("SENS %0.6e" % sens)

    def read_X(self):
        self.pyvisa.write("OUTP? 1")
        return float(self.pyvisa.read())

    def read_Y(self):
        self.pyvisa.write("OUTP? 2")
        return float(self.pyvisa.read())

    def read_R(self):
        self.pyvisa.write("OUTP? 3")
        return float(self.pyvisa.read())

    def read_q(self):
        self.pyvisa.write("OUTP? 4")
        return float(self.pyvisa.read())


# N=40
# Isource_max=400e-9
# Isource1 = np.linspace(0,Isource_max, N+1)
# Isource2 = np.linspace(0, -Isource_max,  N+1)
# Isource3 = np.linspace(-Isource_max, 0,  N+1)
# Isource = np.concatenate([Isource1, Isource2])
# R_srs=2.469135e6
# R_M = 10e9                # R_srs >> Rnw_normal
# V_source = Isource*R_srs
# vread = np.zeros((len(V_source),1))
# iread = np.zeros((len(V_source),1))
# v_d = np.zeros((len(V_source),1))
# i_d = np.zeros((len(V_source),1))

# YOKO.set_voltage(0)
# YOKO.set_output(True)

# voffset = keith.read_voltage()
# # keith.set_output(True)

# sleep(2)
# keith.set_filter(state = 'OFF')
# # ir = - vr / R_srs - 2*vr/R_M


# for k1 in range(len(V_source)):
#     # YOKO.set_voltage(0)
#     # sleep(1)
#     # voffset = keith.read_voltage()

#     YOKO.set_voltage(V_source[k1])
#     if V_source[k1] == 0:
#         sleep(10)
#     sleep(1)
#     vread[k1] = keith.read_voltage()-voffset
#     iread[k1] = (V_source[k1] - vread[k1]) / R_srs - 2*vread[k1]/R_M
#     v_d[k1] = (R_srs/R_M+1)*vread[k1]
#     i_d[k1] = iread[k1]
#     print("I_source :" +str(Isource[k1]*1e9) +" nA, Vd :"+str(v_d[k1]*1e6)+" uV, Rd :"+str(v_d[k1]/i_d[k1])+" ohm")

# YOKO.set_output(False)
# # keith.set_output(False)
# # plt.plot(i_d*1e9,v_d/i_d,'-ob')
# plt.plot(v_d*1e6,i_d*1e9,'-ob')
# temp = ls1.read_temp('D4')
# scipy.io.savemat('S:/SC/Measurements/SPG808/SPG808_IV_340mK_nicelong.mat',{'temp':temp, 'v_device':v_d, 'i_device':i_d, 'v_read':vread})


# pm = Agilent8153A('GPIB0::22')



import os
import sys
import tempfile
import time
from datetime import datetime
from pathlib import Path

import mariadb
import nptdms
import pandas as pd
from mariadb import Connection
from pandas.core.frame import DataFrame

import qnnpy.functions.functions as qf


def load_data_to_database(filename: str, table_name: str, connection: Connection):
    conn = connection
    file_path = os.path.join(filename)
    file_path = file_path.replace("\\", "/")
    cur = conn.cursor()
    command = f"LOAD DATA LOCAL INFILE '{file_path}' IGNORE INTO TABLE `{table_name}` FIELDS TERMINATED BY ',' LINES TERMINATED BY '\\n';"

    try:
        cur.execute(command)
    except mariadb.IntegrityError:
        pass
    conn.commit()


def is_today(file_date: str) -> bool:
    today = datetime.now().strftime("%Y_%m_%d")
    if file_date == today:
        return True
    else:
        return False


def import_tdms(file_path) -> DataFrame:
    try:
        with nptdms.TdmsFile.open(file_path) as tdms_file:
            group = tdms_file["Data"]
            data_dict = {}

            for channel_name in group.channels():
                if channel_name.name in [
                    "Unix Timestamp",
                    "1k",
                    "4k",
                    "50k",
                    "SORB",
                    "Needle Valve 1",
                    "Dump Pressure",
                    "Circulation Pressure",
                ]:
                    channel = group[channel_name.name]
                    data_dict[channel_name.name] = channel[:]

            return format_data(data_dict)

    except FileNotFoundError:
        print("Error: File not found!")


def format_data(data_dict: dict) -> DataFrame:
    df = pd.DataFrame(data_dict)
    df["date_time"] = pd.to_datetime(df["Unix Timestamp"], unit="s")
    df["date_time"] = df["date_time"].dt.strftime("%Y-%m-%d %H:%M:%S")
    df = df.iloc[:, [0, 8, 1, 2, 3, 4, 5, 7, 6]]
    df.columns = [
        "epochtime",
        "datetime",
        "T1",
        "T2",
        "T3",
        "T4",
        "needlevalve",
        "pressure",
        "dump_pressure",
    ]
    return df


def export_to_csv(data_frame: DataFrame):
    with tempfile.NamedTemporaryFile(mode="w", delete=False) as temp:
        data_frame.to_csv(temp.name + ".log", index=False, header=False)
        return temp.name + ".log"


def write_table_to_database(data_frame: DataFrame, table_name: str, connection=None):
    temp_file = export_to_csv(data_frame)
    load_data_to_database(temp_file, table_name, connection)
    Path(temp_file).unlink()
    os.remove(temp_file[:-4])
    return


def files_in_directory(directory: str) -> list:
    file_types = ["tdms"]
    files = []
    for file in os.listdir(directory):
        if file.split(".")[-1] in file_types:
            files.append(file)
    return files


def import_most_recent_tdms(directory: str) -> dict:
    files = files_in_directory(directory)
    files.sort()
    most_recent_file = files[-1]
    file_path = os.path.join(directory, most_recent_file)
    return import_tdms(file_path)


def get_uploaded_files(filename: str) -> list:
    with open(filename, "r") as file:
        files = file.read().splitlines()
    return files


def update_uploaded_files(logfile: str, file: str):
    with open(logfile, "a") as log:
        log.write(file + "\n")
    return


def check_and_import_tdms(directory: str, logfile: str, table_name: str):
    uploaded_files = get_uploaded_files(logfile)
    files = files_in_directory(directory)
    files.sort()

    try:
        conn = qf.database_connection()
        conn.auto_reconnect = True
    except mariadb.Error as e:
        print(f"Error connecting to the database: {e}")
        sys.exit(1)

    for file in files:
        if file not in uploaded_files:
            print(f"Importing: {file}")
            file_path = os.path.join(directory, file)
            file_date = file[0:10]

            data_frame = import_tdms(file_path)
            write_table_to_database(data_frame, table_name, conn)

            update_uploaded_files(logfile, file)
            time.sleep(5)

    conn.close()
    return


if __name__ == "__main__":
    LOG_DIRECTORY = r"S:\SC\InstrumentLogging\Cryogenics\Ice\ice-log\Logs"
    LOG_FILE = r"S:\SC\InstrumentLogging\Cryogenics\Ice\ice-log\uploaded_files.txt"
    TABLE_NAME = "ice_log"

    check_and_import_tdms(LOG_DIRECTORY, LOG_FILE, TABLE_NAME)


# --------------------------------------------------------------------------------------------
# Archived functions for ICE Software V2.0
# --------------------------------------------------------------------------------------------

# def log_many_rows_to_database(table_name: str, connection=None, **kwargs):
#     """
#     table_name [str]: name of table to insert rows to
#     kwargs: should be of form COLUMN_NAME:[list, of, values, in, column]

#     executemany takes two arguments:
#         -1) the command string, with VALUES (%s, %s...%s) for #columns to write
#         -2) a list of tuples of rows, e.g.
#             data = [
#                 (row1_column1, row1_column2, ...row1_columnn),
#                 ...
#                 (rowm_column1, rowm_column2, ...rowm_columnn)
#             ]
#     """
#     if connection is not None:
#         conn = connection
#     else:
#         conn = qf.database_connection()

#     cur = conn.cursor()
#     column_names = "`" + "`, `".join(kwargs.keys()) + "`"
#     values_sideways = array([v for v in kwargs.values()])
#     values = values_sideways.T.tolist()
#     # values = "'"+"', '".join([str(v) for v in kwargs.values()])+"'"
#     for value in values:
#         try:
#             command = "INSERT INTO `%s` (%s) VALUES (%s)" % (
#                 table_name,
#                 column_names,
#                 str(value)[1:-1],
#             )
#             cur.execute(command)
#             conn.commit()
#             time.sleep(0.5)
#         except mariadb.IntegrityError:
#             pass
#     if connection == None:
#         conn.close()


# def read_ice_log(path, date=False):
#     log_columns = {
#         "date": [],
#         "time": [],
#         "epochtime": [],
#         "date_time": [],
#         "T1": [],
#         "T2": [],
#         "T3": [],
#         "T4": [],
#         "sensor1": [],
#         "sensor2": [],
#         "sensor3": [],
#         "sensor4": [],
#         "heater1": [],
#         "heater2": [],
#         "needlevalve": [],
#         "needlevalve2": [],
#         "null1": [],
#         "null2": [],
#         "setpoint2": [],
#         "20_1": [],
#         "ramprate2": [],
#         "null3": [],
#         "P": [],
#         "I": [],
#         "D": [],
#         "50_1": [],
#         "20_2": [],
#         "null4": [],
#         "null5": [],
#         "dump_pressure": [],
#     }

#     datetime_format = "%m/%d/%Y %I:%M:%S %p"

#     if not date:
#         # yesterday = datetime.now() - timedelta(1)
#         date = datetime.strftime(datetime.now(), "%Y-%m-%d")

#     path = path + date + "/"

#     try:
#         with open(path + date + ".log") as csvfile:
#             read = csv.reader(csvfile)

#             with open(path + "temp.log", "w", newline="\n") as csvfile:
#                 write = csv.writer(csvfile, delimiter=",")

#                 for i, row in enumerate(read):
#                     log_columns["epochtime"].append(int(row[2]))
#                     datetime_data = datetime.strftime(
#                         datetime.strptime(row[3], datetime_format), "%Y-%m-%d %H:%M:%S"
#                     )
#                     log_columns["date_time"].append(datetime_data)
#                     log_columns["T1"].append(float(row[4]))
#                     log_columns["T2"].append(float(row[5]))
#                     log_columns["T3"].append(float(row[6]))
#                     log_columns["T4"].append(float(row[7]))
#                     log_columns["needlevalve"].append(float(row[14]))
#                     log_columns["dump_pressure"].append(float(row[29]))

#                     write.writerow(
#                         [
#                             log_columns["epochtime"][i],
#                             log_columns["date_time"][i],
#                             log_columns["T1"][i],
#                             log_columns["T2"][i],
#                             log_columns["T3"][i],
#                             log_columns["T4"][i],
#                             log_columns["needlevalve"][i],
#                             log_columns["dump_pressure"][i],
#                         ]
#                     )
#     except Exception as e:
#         print(e)
#         time.sleep(600)

#     load_data_to_database(path + "temp.log", "ice_log")
#     file_to_rem = Path(path + "temp.log")
#     file_to_rem.unlink()
#     return



import numpy as np
import pyvisa


class HP8722C(object):
    """Python class for HP 8722C Network Analyzer, written by Adam McCaughan"""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def reset(self):
        self.write("*RST")
        self.timeout = 5

    def freq_range(
        self, f_start=0.1e9, f_stop=1.0e9, f_center=None, f_span=None, num_pts=401
    ):
        if (f_center is not None) and (f_span is not None):
            f_start = f_center - f_span / 2.0
            f_stop = f_center + f_span / 2.0

        self.write("POIN %0.0i;" % num_pts)
        self.write("STAR %0.6e;" % f_start)
        self.write("STOP %0.6e;" % f_stop)

    def fixed_freq(self, f=10e6):
        self.write("CWFREQ%0.6eHZ;" % f)

    def power(self, power=-20):
        if (power < -60) or (power > 0):
            print("Out of range value")
            return
        if power >= -5:
            power_range = "01"  # Only available below 26 GHz
        elif power >= -20:
            power_range = "02"
        elif power >= -35:
            power_range = "05"
        elif power >= -50:
            power_range = "08"
        elif power >= -60:
            power_range = "10"

        self.write("PRAN%s" % power_range)
        self.write("POWE %0.0d" % power)  # Sets power (in dBm)

    def s_mode(self, s_mode="S11"):
        self.write("%s;" % s_mode)

    def format_polar(self):
        self.write("POLA")  # Set to polar coordinates

    def format_logarithmic(self):
        self.write("LOGM")  # Set to polar coordinates

    def run_sweep_ri(self):
        """Runs a sweep using whatever settings are currently on the NA and returns the real
        and imaginary components of each data point"""
        self.format_polar()  # Set to polar coordinates
        f_start = float(self.query("STAR?;"))
        f_span = float(self.query("SPAN?;"))
        f_stop = float(self.query("STOP?;"))
        num_pts = int(float(self.query("POIN?;")))

        print(
            "Sweeping from %0.0d MHz to %0.0d MHz, with %0.0d points"
            % (f_start / 1e6, f_stop / 1e6, num_pts)
        )

        temp = self.timeout
        self.timeout = 20
        completion = self.query(
            "OPC?;SING;"
        )  # Runs a SINGle sweep, and waits for the OPeration to Complete
        self.timeout = temp

        self.write("FORM4;")  # Make the data output in ASCII
        data = self.ask_for_values("OUTPFORM;")

        freq = np.linspace(f_start, f_stop, num_pts, endpoint=True)
        real = data[::2]  # Every other element starting with element 0
        imag = data[1::2]
        self.write("CONT")
        return freq, real, imag

    def run_sweep_mag(self):
        """Runs a sweep using whatever settings are currently on the NA and returns the real
        and imaginary components of each data point"""
        self.format_logarithmic()  # Set to logarithimic coordinates
        f_start = float(self.query("STAR?;"))
        f_span = float(self.query("SPAN?;"))
        f_stop = float(self.query("STOP?;"))
        num_pts = int(float(self.query("POIN?;")))

        print(
            "Sweeping from %0.0d MHz to %0.0d MHz, with %0.0d points"
            % (f_start / 1e6, f_stop / 1e6, num_pts)
        )

        temp = self.timeout
        self.timeout = 20
        completion = self.query(
            "OPC?;SING;"
        )  # Runs a SINGle sweep, and waits for the OPeration to Complete
        self.timeout = temp

        self.write("FORM4;")  # Make the data output in ASCII
        data = self.ask_for_values("OUTPFORM;")

        f = np.linspace(f_start, f_stop, num_pts, endpoint=True)
        F = f[::1]
        M = data[::2]  # Every other element starting with element 0
        self.write("CONT")
        return F, M

    def run_sweep_ri_logspace(self):
        """Runs a sweep using whatever settings are currently on the NA and returns the real
        and imaginary components of each data point"""
        self.format_polar()  # Set to polar coordinates
        f_start = float(self.query("STAR?;"))
        f_span = float(self.query("SPAN?;"))
        f_stop = float(self.query("STOP?;"))
        num_pts = int(float(self.query("POIN?;")))

        print(
            "Sweeping from %0.0d MHz to %0.0d MHz, with %0.0d points"
            % (f_start / 1e6, f_stop / 1e6, num_pts)
        )

        temp = self.timeout
        self.timeout = 20
        completion = self.query(
            "OPC?;SING;"
        )  # Runs a SINGle sweep, and waits for the OPeration to Complete
        self.timeout = temp

        self.write("FORM4;")  # Make the data output in ASCII
        data = self.ask_for_values("OUTPFORM;")

        freq = np.logspace(np.log10(f_start), np.log10(f_stop), num_pts, endpoint=True)
        real = data[::2]  # Every other element starting with element 0
        imag = data[1::2]
        self.write("CONT")
        return freq, real, imag



import csv
#import schedule
import time
from datetime import datetime, timedelta
import mariadb
from numpy import array
from pathlib import Path

import sys
sys.path.append(r'Q:\qnnpy')
import qnnpy.functions.functions as qf

PATH = 'S:/SC/InstrumentLogging/Cryogenics/Ice/ice-log/Results/'

# Set the window title
if sys.platform.startswith('win32'):
    import ctypes
    ctypes.windll.kernel32.SetConsoleTitleW("data_logging.py")

def load_data_to_database(filename: str, table_name: str, connection=None):
    if connection is not None:
        conn=connection 
    else:
        conn = qf.database_connection()

    cur = conn.cursor()
    command = "LOAD DATA LOCAL INFILE '%s' IGNORE INTO TABLE `%s` FIELDS TERMINATED BY ',' LINES TERMINATED BY '\n' " % (filename, table_name) 
    cur.execute(command)
    conn.commit()
    time.sleep(1)
    conn.close()
    

def log_many_rows_to_database(table_name: str, connection = None, **kwargs):
    '''
    table_name [str]: name of table to insert rows to
    kwargs: should be of form COLUMN_NAME:[list, of, values, in, column]

    executemany takes two arguments:
        -1) the command string, with VALUES (%s, %s...%s) for #columns to write
        -2) a list of tuples of rows, e.g. 
            data = [
                (row1_column1, row1_column2, ...row1_columnn),
                ...
                (rowm_column1, rowm_column2, ...rowm_columnn)
            ]
    '''
    if connection is not None:
        conn=connection 
    else:
        conn = qf.database_connection()
    
    cur = conn.cursor()
    column_names = "`"+"`, `".join(kwargs.keys())+"`"
    values_sideways = array([v for v in kwargs.values()])
    values = values_sideways.T.tolist()
    #values = "'"+"', '".join([str(v) for v in kwargs.values()])+"'"
    for value in values:
        try:
            command = "INSERT INTO `%s` (%s) VALUES (%s)" % (table_name, column_names, str(value)[1:-1])
            cur.execute(command)
            conn.commit()
            time.sleep(0.1)
        except mariadb.IntegrityError:
            pass
    if connection==None:
        conn.close()

def read_ice_log(path, date=False):

    log_columns = {
        'date':[],
        'time':[],
        'epochtime':[],
        'date_time':[],
        'T1':[],
        'T2':[],
        'T3':[],
        'T4':[],
        'sensor1':[],
        'sensor2':[],
        'sensor3':[],
        'sensor4':[],
        'heater1':[],
        'heater2':[],
        'needlevalve':[],
        'needlevalve2':[],
        'null1':[],
        'null2':[],
        'setpoint2':[],
        '20_1':[],
        'ramprate2':[],
        'null3':[],
        'P':[],
        'I':[],
        'D':[],
        '50_1':[],
        '20_2':[],
        'null4':[],
        'null5':[],
        'dump_pressure':[]
    }

    datetime_format = '%m/%d/%Y %I:%M:%S %p'

    if not date:
        #yesterday = datetime.now() - timedelta(1)
        date = datetime.strftime(datetime.now(), '%Y-%m-%d')
        
    path = path+date+'/'
    
    try:
    
        with open(path+date+'.log') as csvfile:
            read = csv.reader(csvfile)
    
            with open(path+'temp.log', 'w', newline='\n') as csvfile:
                write = csv.writer(csvfile, delimiter=',')
    
    
                for i, row in enumerate(read):
                    log_columns['epochtime'].append(int(row[2]))
                    datetime_data = datetime.strftime(datetime.strptime(row[3], datetime_format), '%Y-%m-%d %H:%M:%S')
                    log_columns['date_time'].append(datetime_data)
                    log_columns['T1'].append(float(row[4]))
                    log_columns['T2'].append(float(row[5]))
                    log_columns['T3'].append(float(row[6]))
                    log_columns['T4'].append(float(row[7]))
                    log_columns['needlevalve'].append(float(row[14]))
                    log_columns['dump_pressure'].append(float(row[29]))
    
                    write.writerow(
                        [log_columns['epochtime'][i],
                        log_columns['date_time'][i],
                        log_columns['T1'][i],
                        log_columns['T2'][i],
                        log_columns['T3'][i],
                        log_columns['T4'][i],
                        log_columns['needlevalve'][i],
                        log_columns['dump_pressure'][i]]
                    )
    except Exception as e:
        print(e)
        time.sleep(600)

    load_data_to_database(path+'temp.log', 'ice_log')  
    file_to_rem = Path(path+'temp.log')
    file_to_rem.unlink()
    return


#schedule.every().day.at('01:00').do(read_ice_log, path=PATH)
#schedule.every(10).minutes.do(read_ice_log, path=PATH)

while True:
    read_ice_log(PATH)
    time.sleep(60)


import csv
import sys
import os
import atexit 
import logging
import signal
import time
from ice_logging import read_ice_log
sys.path.append(r'Q:\qnnpy')
import qnnpy.functions.functions as qf


PATH = "S:/SC/InstrumentLogging/Cryogenics/Ice/ice-log/Results/"

# Setup logging
log_file = r'C:\Users\ICE\ice-logging\logfile.txt'
logging.basicConfig(filename=log_file, level=logging.INFO, format='%(asctime)s %(message)s')

logging.info('Script started.')


# Define the flag file path
flag_file = r'C:\Users\ICE\ice-logging\flagfile.txt'

# Create the flag file
try:
    with open(flag_file, 'w') as f:
        f.write('Running')
        logging.info('Flag file created.')
except Exception as e:
    logging.error(f'Error creating flag file: {e}')

    
# Register the flag file for removal on exit
def remove_flag_file():
    if os.path.exists(flag_file):
        os.remove(flag_file)

atexit.register(remove_flag_file)

def signal_handler(sig, frame):
    logging.info('Signal received: %s. Cleaning up and exiting.', sig)
    remove_flag_file()
    sys.exit(0)

signal.signal(signal.SIGINT, signal_handler)
signal.signal(signal.SIGTERM, signal_handler)




try:
    while True:
        read_ice_log(PATH)
        time.sleep(120)
        pass
except KeyboardInterrupt:
    logging.info('KeyboardInterrupt received. Exiting.')
finally:
    logging.info('Script ended.')


# -*- coding: utf-8 -*-
"""
Created on Mon Jan 23 10:38:22 2023

@author: omedeiro
"""

import sys
from time import sleep

sys.path.append(r"Q:\qnnpy")
import qnnpy.functions.functions as qf
from qnnpy.instruments.lakeshore336 import Lakeshore336

# https://stackoverflow.com/questions/59125493/how-to-constantly-run-python-script-in-the-background-on-windows


while True:
    try:
        ls1 = Lakeshore336("GPIB0::13::INSTR")
    except Exception:
        print("failed connection to Lakeshore336")

    try:
        stage_3K = ls1.read_temp("D4")
        sleep(2)
        stage_40K = ls1.read_temp("D2")
        sleep(2)
        stage_SORB = ls1.read_temp("D3")
        sleep(2)
        stage_He3Pot = ls1.read_temp("D1")
        sleep(1)
        # print([stage_3K, stage_40K, stage_SORB, stage_He3Pot])
    except Exception:
        print("failed to read temp")
    # ls1.read_temp('D')

    try:
        qf.log_data_to_database(
            table_name="janis_300mK_logging",
            stage_3K=stage_3K,
            stage_40K=stage_40K,
            stage_SORB=stage_SORB,
            stage_He3Pot=stage_He3Pot,
        )
    except Exception:
        print("failed to write to database")

    sleep(1)



from time import sleep

import numpy as np

import qnnpy.functions.functions as qf

# note: instrument libraries are imported within the class as needed from config file


class Resonators:
    """
    class for measuring superconducting resonators
    """

    def __init__(self, config_file):
        # Load config file
        self.properties = qf.load_config(config_file)

        self.sample_name = self.properties["Save File"]["sample name"]
        self.device_name = self.properties["Save File"]["device name"]
        self.device_type = self.properties["Save File"]["device type"]
        self.R_srs = self.properties["iv_sweep"]["series_resistance"]

        self.instrument_list = []

        ### Set up instruments ###
        if self.properties.get("PNA"):
            self.instrument_list.append("PNA")

            if self.properties["PNA"]["name"] == "KeysightN5224a":
                from qnnpy.instruments.keysight_n5224a import KeysightN5224a

                try:
                    self.pna = KeysightN5224a(self.properties["PNA"]["port"])
                    # self.pna.set_power(0)
                    self.pna.set_scale_auto()
                    print("PNA: connected")
                except Exception:
                    print("PNA: failed to connect")
            else:
                qf.lablog(
                    'Invalid PNA. PNA name: "%s" is not'
                    "configured" % self.properties["PNA"]["name"]
                )
                raise NameError("Invalide PNA. PNA name is not configured")

        # Meter
        if self.properties.get("Meter"):
            self.instrument_list.append("Meter")

            if self.properties["Meter"]["name"] == "Keithley2700":
                from qnnpy.instruments.keithley_2700 import Keithley2700

                try:
                    self.meter = Keithley2700(self.properties["Meter"]["port"])
                    self.meter.reset()
                    print("METER: connected")
                except Exception:
                    print("METER: failed to connect")

            elif self.properties["Meter"]["name"] == "Keithley2400":
                # this is a source meter
                from qnnpy.instruments.keithley_2400 import Keithley2400

                try:
                    self.meter = Keithley2400(self.properties["Meter"]["port"])
                    self.meter.reset()
                    print("METER: connected")
                except Exception:
                    print("METER: failed to connect")

            elif self.properties["Meter"]["name"] == "Keithley2001":
                from qnnpy.instruments.keithley_2001 import Keithley2001

                try:
                    self.meter = Keithley2001(self.properties["Meter"]["port"])
                    self.meter.reset()
                    print("METER: connected")
                except Exception:
                    print("METER: failed to connect")
            else:
                qf.lablog(
                    'Invalid Meter. Meter name: "%s" is not configured'
                    % self.properties["Meter"]["name"]
                )
                raise NameError(
                    'Invalid Meter. Meter name: "%s" is not configured'
                    % self.properties["Meter"]["name"]
                )

        # Source
        if self.properties.get("Source"):
            self.instrument_list.append("Source")

            if self.properties["Source"]["name"] == "SIM928":
                from qnnpy.instruments.srs_sim928 import SIM928

                try:
                    self.source = SIM928(
                        self.properties["Source"]["port"],
                        self.properties["Source"]["port_alt"],
                    )
                    self.source.reset()

                    try:
                        self.properties.get("Source")["port_alt2"]
                        self.source2 = SIM928(
                            self.properties["Source"]["port"],
                            self.properties["Source"]["port_alt2"],
                        )
                    except Exception:
                        print("No second SRS specified")

                    print("SOURCE: connected")
                except Exception:
                    print("SOURCE: failed to connect")
            else:
                qf.lablog(
                    'Invalid Source. Source name: "%s" is not configured'
                    % self.properties["Source"]["name"]
                )
                raise NameError(
                    'Invalid Source. Source name: "%s" is not configured'
                    % self.properties["Source"]["name"]
                )

        # Temperature Controller
        if self.properties.get("Temperature"):
            self.instrument_list.append("Temperature")

            if self.properties["Temperature"]["name"] == "Cryocon350":
                from qnnpy.instruments.cryocon350 import Cryocon350

                try:
                    self.temp = Cryocon350(self.properties["Temperature"]["port"])
                    self.temp.channel = self.properties["Temperature"]["channel"]
                    self.properties["Temperature"]["initial temp"] = (
                        self.temp.read_temp(self.temp.channel)
                    )
                    print("TEMPERATURE: connected")
                except Exception:
                    print("TEMPERATURE: failed to connect")

            elif self.properties["Temperature"]["name"] == "Cryocon34":
                from qnnpy.instruments.cryocon34 import Cryocon34

                try:
                    self.temp = Cryocon34(self.properties["Temperature"]["port"])
                    self.temp.channel = self.properties["Temperature"]["channel"]
                    self.properties["Temperature"]["initial temp"] = (
                        self.temp.read_temp(self.temp.channel)
                    )
                    print("TEMPERATURE: connected")
                except Exception:
                    print("TEMPERATURE: failed to connect")

            elif self.properties["Temperature"]["name"] == "ICE":
                try:
                    self.properties["Temperature"]["initial temp"] = qf.ice_get_temp(
                        select=1
                    )
                    print("TEMPERATURE: connected")
                except Exception:
                    print("TEMPERATURE: failed to connect")

            elif self.properties["Temperature"]["name"] == "DEWAR":
                try:
                    self.properties["Temperature"]["initial temp"] = 4.2
                    print("TEMPERATURE: ~connected~ 4.2K")
                except Exception:
                    print("TEMPERATURE: failed to connect")

            else:
                qf.lablog(
                    'Invalid Temperature Controller. TEMP name: "%s" is not configured'
                    % self.properties["Temperature"]["name"]
                )
                raise NameError(
                    'Invalid Temperature Controller. TEMP name: "%s" is not configured'
                    % self.properties["Temperature"]["name"]
                )
        else:
            self.properties["Temperature"] = {"initial temp": "None"}
            # print('TEMPERATURE: Not Specified')

    def configure_pna(
        self, start_freq, stop_freq, num_points, power=0, if_bandwidth=100
    ):
        # self.pna.reset(measurement='S21', if_bandwidth = 20, start_freq = 200E6,
        #     stop_freq = 20E9, power = -40)

        self.pna.set_start(start_freq)
        self.pna.set_stop(stop_freq)
        self.pna.set_points(num_points)
        self.pna.set_power(power)
        self.pna.set_if_bw(if_bandwidth)
        sleep(5)
        self.pna.set_scale_auto()

    def S21_measurement(
        self, start_freq, stop_freq, num_points, power=0, if_bandwidth=100
    ):
        """
        measure the S21 data on a superconducting resonator using a VNA
        start_freq, stop_freq = start/end points of frequency sweep (Hz)
        num_points = number of points in frequenyc sweep
        if_bandwidth = if_bandwidth (Hz)
        power = output power of VNA (dBm)

        class stores the measured source attenuation, the frequency points
        swept over, and resulting S21 values
        """
        self.configure_pna(start_freq, stop_freq, num_points, power, if_bandwidth)

        self.f, self.S21 = self.pna.single_sweep()
        self.att = self.pna.get_source_attenuation()

    def phase_measurement(
        self, start_freq, stop_freq, num_points, power=0, if_bandwidth=100
    ):
        self.configure_pna(start_freq, stop_freq, num_points, power, if_bandwidth)

        self.f, self.phase = self.pna.single_sweep_phase()
        self.att = self.pna.get_source_attenuation()

    def power_characterization(
        self, start_freq, stop_freq, num_points, powers, if_bandwidth=100
    ):
        for power in powers:
            self.S21_measurement(start_freq, stop_freq, num_points, power, if_bandwidth)
            self.save()
            self.plot()

    def S21_to_dBm(self):
        return 20 * np.log10(np.abs(self.S21))

    def plot(self, close=True):
        full_path = qf.save(self.properties, "S21_measurement")
        qf.plot(
            self.f,
            self.S21_to_dBm(),
            title=self.sample_name
            + " "
            + self.device_type
            + " "
            + self.device_name
            + " "
            + str(self.pna.get_power())
            + "dB",
            xlabel="Frequency (Hz)",
            ylabel="S21 (dBm)",
            path=full_path,
            close=close,
        )

    def save(self):
        data_dict = {
            "S21": self.S21_to_dBm(),
            "pna_power": self.pna.get_power(),
            "f": self.f,
            "pna_att": self.att,
        }
        self.full_path = qf.save(
            self.properties,
            "S21_measurement",
            data_dict,
            instrument_list=self.instrument_list,
        )


class IvSweep(Resonators):
    """Class object for iv sweeps

    Configuration: iv_sweep:
    [start: initial bias current],
    [stop: final bias current],
    [steps: number of points],
    [sweep: number of itterations],
    [full_sweep: Include both positive and negative bias],
    [series_resistance: resistance at voltage source ie R_srs]

    """

    def run_sweep_fixed(self):
        """Runs IV sweep with config paramters
        This constructs #steps between start and 75% of final current.
        Then, #steps between 75% and 100% of final current.

        np.linspace()
        """
        self.source.reset()
        self.meter.reset()

        self.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        steps = self.properties["iv_sweep"]["steps"]
        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop * 0.75, steps)  # Coarse
        Isource2 = np.linspace(stop * 0.75, stop, steps)  # Fine

        if full_sweep:
            Isource = np.concatenate(
                [Isource1, Isource2, Isource2[::-1], Isource1[::-1]]
            )
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource1, Isource2])
        self.v_set = np.tile(Isource, sweep) * self.R_srs

        self.source.set_output(True)
        sleep(1)
        voltage = []
        current = []

        for n in self.v_set:
            self.source.set_voltage(n)
            sleep(0.1)

            vread = self.meter.read_voltage()  # Voltage

            iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def run_sweep_dynamic(self):
        """Runs IV sweep with config paramters
        This constructs [points_coarse] between start and [percent] of final current.
        Then, [points_fine] between [percent] and 100% of final current.

        uses np.linspace()
        """
        self.source.reset()
        self.meter.reset()

        self.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        percent = self.properties["iv_sweep"]["percent"]
        num1 = self.properties["iv_sweep"]["points_coarse"]
        num2 = self.properties["iv_sweep"]["points_fine"]

        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop * percent, num1)  # Coarse
        Isource2 = np.linspace(stop * percent, stop, num2)  # Fine

        if full_sweep:
            Isource = np.concatenate(
                [Isource1, Isource2, Isource2[::-1], Isource1[::-1]]
            )
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource1, Isource2])
        self.v_set = np.tile(Isource, sweep) * self.R_srs

        self.source.set_output(True)
        sleep(1)
        voltage = []
        current = []

        for n in self.v_set:
            self.source.set_voltage(n)
            sleep(0.1)

            vread = self.meter.read_voltage()  # Voltage

            iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def run_sweep_spacing(self):
        """Runs IV sweep with config paramters
        This constructs [points_coarse] between start and [percent] of final current.
        Then, [points_fine] between [percent] and 100% of final current.

        uses np.arange()
        """
        self.source.reset()
        self.meter.reset()

        self.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        percent = self.properties["iv_sweep"]["percent"]
        spacing1 = self.properties["iv_sweep"]["spacing_coarse"]
        spacing2 = self.properties["iv_sweep"]["spacing_fine"]

        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.arange(start, stop * percent + spacing1, spacing1)  # Coarse
        Isource2 = np.arange(stop * percent, stop + spacing2, spacing2)  # Fine

        if full_sweep:
            Isource = np.concatenate(
                [Isource1, Isource2, Isource2[::-1], Isource1[::-1]]
            )
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource1, Isource2])
        self.v_set = np.tile(Isource, sweep) * self.R_srs

        self.source.set_output(True)
        sleep(1)
        voltage = []
        current = []

        for n in self.v_set:
            self.source.set_voltage(n)
            sleep(0.1)

            vread = self.meter.read_voltage()  # Voltage

            iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def isw_calc(self):
        """Calculates critical current."""
        """
        Critical current is defined as the current one step before a voltage
        reading greater than 5mV.
        self.isw = self.i_read[np.argmax(np.array(self.v_read) > 0.005)-1]
        print('%.4f uA' % (self.isw*1e6))
        """

        """ New method looks at differential and takes average of points that 
        meet condition. Prints mean() and std().` std should be < 1-2µA
        """
        # isws = abs(self.i_read[np.where(abs(np.diff(self.i_read)/max(np.diff(self.i_read))) > 0.5)])
        # self.isw = self.i_read[np.argmax(np.array(self.v_read) > 0.005)-1]
        # print('%.4f uA' % (self.isw*1e6))

        try:
            switches = np.asarray(
                np.where(abs(np.diff(self.i_read) / max(np.diff(self.i_read))) > 0.8),
                dtype=int,
            ).squeeze()
            isws = abs(np.asarray([self.i_read[i] for i in switches]))
            print(isws)
            self.isw = isws.mean()
            print(
                "Isw_avg = %.4f µA :--: Isw_std = %.4f µA"
                % (self.isw * 1e6, isws.std() * 1e6)
            )
        except Exception:
            print("Could not calculate Isw. Isw set to 0")
            self.isw = 0

    def plot(self):
        full_path = qf.save(self.properties, "iv_sweep")
        qf.plot(
            np.array(self.v_read),
            np.array(self.i_read) * 1e6,
            title=self.sample_name + " " + self.device_type + " " + self.device_name,
            xlabel="Voltage (mV)",
            ylabel="Current (uA)",
            path=full_path,
            show=True,
            close=True,
        )

    def save(self):
        # Set up data dictionary

        data_dict = {
            "V_source": self.v_set,
            "V_device": self.v_read,
            "I_device": self.i_read,
            "Isw": self.isw,
            "R_srs": self.R_srs,
            "temp": self.properties["Temperature"]["initial temp"],
        }

        self.full_path = qf.save(
            self.properties, "iv_sweep", data_dict, instrument_list=self.instrument_list
        )



"""
Laser control scripts for Santec tunable laser
Scripts from the Loncar group at Harvard.
Consider deleting if not needed.
"""

import os
import time

import nidaqmx
import numpy as np
from nidaqmx.constants import TaskMode

from ..santec_tsl import SantecTSL

# import ..models.modeltools as modeltools


class LaserCtrl:
    """LaserCtrl uses a laser and photodiode to perform monitor and scan
    transmission data.
    """

    def __init__(self):
        self.laser = None  # laser object
        self.config = {}
        self.stop = False
        self.loop_scanning = False
        self.most_recent_scan = "One way"
        self.sweep_task = None
        arr0 = np.array(
            [
                0.0,
            ]
        )
        # initialize empty datasets
        self.scandata = {
            "wav": arr0,
            "V": arr0,
        }
        self.scandata2way = {
            "wavstartstop": arr0,
            "Vstartstop": arr0,
            "wavstopstart": arr0,
            "Vstopstart": arr0,
        }
        self.monitordata = [
            0,
        ]
        self.monitor_nsteps = 10  # number of points to scroll
        path_here = os.path.dirname(__file__)
        self._default_config_path = os.path.join(
            path_here, "config_defaults/laserctrl_default.yml"
        )
        self.loaded_config_path = None

    def load_config(self, fpath=None):
        # load YAML file from fpath or default if fpath=None
        self.config = modeltools.load_config(
            fpath, defaultpath=self._default_config_path
        )
        if fpath is None:
            self.loaded_config_path = self._default_config_path
        else:
            self.loaded_config_path = fpath

    def overwrite_config(self):
        # overwrite YAML file used to load_config with current config
        modeltools.save_config(self.loaded_config_path, self.config)

    def load_laser(self, laser=None):
        # load laser, if None given, autoload from config
        if laser is None:
            kind = self.config["Laser"]["kind"]
            rsc_name = self.config["Laser"]["rsc_name"]
            if kind == "SantecTSLDummy":
                self.laser = SantecTSLDummy(rsc_name)
            elif kind == "SantecTSL":
                self.laser = SantecTSL(rsc_name)
            else:
                raise Exception("Laser kind not recoginized in config")
            self.laser.initialize()
        else:
            self.laser = laser
            self.config["Laser"]["kind"] = str(self.laser)
            self.config["Laser"]["rsc_name"] = self.laser.rsc_name

    def monitor(self):
        """Monitor photoreceiver voltage in time.
        Config Args:
            Monitor.tstep - timestep in sec, float
            Monitor.scroll_time - length of stored data in sec, float
        Updates:
            monitordata - voltage measurements, 1D list
        Notes:
            + This function runs forever. You'll want to run it in a thread
            + Can be stopped during acquisition by self.stop = True
        """
        print("Starting monitoring")
        scroll_time = self.config["Monitor"]["scroll_time"]
        tstep = self.config["Monitor"]["tstep"]
        self.monitor_nsteps = int(scroll_time / tstep)
        # setup analog read daq task
        daqtask = nidaqmx.Task()
        daqtask.ai_channels.add_ai_voltage_chan(self.config["DAQ"]["ai"])
        # init data to empty list
        self.monitordata = []
        self.stop = False
        while not self.stop:
            time.sleep(tstep)
            datum = daqtask.read()  # next measurement
            if len(self.monitordata) < self.monitor_nsteps:
                self.monitordata = self.monitordata + [datum]
            else:
                self.monitordata[:-1] = self.monitordata[1:]
                self.monitordata[-1] = datum
        self.stop = False  # reset stop signal
        daqtask.close()

    def wav_valid(self, wav):
        return self.laser.wav_min <= wav <= self.laser.wav_max

    def _single_scan(self, backwards=False):
        # private single scan function, returns scandata
        scandata = {}
        if not backwards:
            wavstart = self.config["Scan"]["wavstart"]
            wavstop = self.config["Scan"]["wavstop"]
        else:
            wavstart = self.config["Scan"]["wavstop"]
            wavstop = self.config["Scan"]["wavstart"]
        samples = self.config["Scan"]["samples"]
        speed = self.config["Scan"]["speed"]
        t_acq = np.abs(wavstart - wavstop) / speed  # expected acquisition time
        print("tacq: {:.3}".format(t_acq))
        sample_rate = float(samples / t_acq)
        wavs = np.linspace(wavstart, wavstop, samples)
        scandata["wav"] = wavs

        # configure daq task
        self.sweep_task = nidaqmx.Task()
        self.sweep_task.ai_channels.add_ai_voltage_chan(self.config["DAQ"]["ai"])
        max_sample_rate = self.sweep_task.timing.samp_clk_max_rate
        if sample_rate > max_sample_rate:
            self.sweep_task.close()
            msg = "The requested sample rate {} is faster than the max rate {}".format(
                sample_rate, max_sample_rate
            )
            msg += ". Adjust scan settings to lower sample rate."
            raise Exception(msg)
        self.sweep_task.timing.cfg_samp_clk_timing(
            float(sample_rate), samps_per_chan=samples
        )
        self.sweep_task.triggers.reference_trigger.cfg_dig_edge_ref_trig(
            trigger_source=self.config["DAQ"]["trig"], pretrigger_samples=2
        )
        # run scan
        self.sweep_task.start()
        try:
            self.laser.setStartWavelength(wavstart)
            self.laser.setStopWavelength(wavstop)
            self.laser.setSweepSpeed(speed)
            self.laser.setSweepMode(1)
            self.laser.setSweepCycles(numOfSweeps=1)
            self.laser.setTriggerOut(triggerOut=True)
            self.laser.startSweep()
            timeout = self.config["DAQ"]["timeout"]
            voltages = self.sweep_task.read(samples, timeout=t_acq + timeout)
            scandata["wav"] = wavs
            scandata["V"] = voltages
        except nidaqmx.errors.DaqError:
            print("Daq error in LaserCtrl._single_scan")
            # scandata['V'] = np.zeros(len(wavs))
            scandata = self.scandata
        self.sweep_task.close()
        return scandata

    def stop_scan(self):
        # abort scan mid-way
        self.laser.stopSweep()
        self.sweep_task.control(TaskMode.TASK_ABORT)
        print("Scan aborted")

    def scan(self, backwards=False):
        """Perform single continuous wavelength scan
        Args:
            backwards=False: if True, scan from wavstop to wavstart
        Config Args:
            Scan.wavestart - start wavelength in nm, float
            Scan.wavstop - stop wavelength in nm, float
            Scan.samples - number of samples to collect, int
            Scan.speed - scan speed in nm/s
        Updates:
            scandata['wav'] - measured wavelengths in nm, array of float
            scandata['V'] - measured voltages in V, array of float
        Notes:
            Wavelengths are estimated by assuming linear scan in time
        """
        print("Doing a one-way scan!")
        self.scandata = self._single_scan(backwards=backwards)
        self.most_recent_scan = "One way"
        if not self.loop_scanning:
            time.sleep(0.2)
            self.laser.setWavelength(self.config["LaserSettings"]["wavelength"])

    def scan_twoway(self):
        """Perform forward(start>stop) scan, then backwards(stop>start) scan
        Config Args:
            Same as LaserCtrl.scan() above
        Updates:
            scandata['wavstartstop'] - wavelengths in nm, array of float
            scandata['wavstopstart'] - wavelengths in nm, array of float
            scandata['Vstartstop'] - measured voltages in V, array of float
            scandata['Vstopstart'] - measured voltages in V, array of float
        """
        self.stop = False
        print("Doing a two-way scan!")
        data_temp = {}
        # forward scan
        scandata = self._single_scan(backwards=False)
        data_temp["wavstartstop"] = scandata["wav"]
        data_temp["Vstartstop"] = scandata["V"]
        time.sleep(0.3)
        # backward scan
        if not self.stop:
            scandata = self._single_scan(backwards=True)
            data_temp["Vstopstart"] = scandata["V"]
        else:
            data_temp["Vstopstart"] = np.zeros(len(scandata["wav"]))
        data_temp["wavstopstart"] = scandata["wav"]
        # update data
        self.scandata2way = data_temp
        self.most_recent_scan = "Two way"
        if not self.loop_scanning:
            time.sleep(0.2)
            self.laser.setWavelength(self.config["LaserSettings"]["wavelength"])

    def scan_loop(self, two_way=False):
        """Perform scan in a loop
        Config Args:
            same as LaserCtrl.scan
            Scan.loop_delay - wait time between scans
        Args:
            two_way=False - do two-way scan using LaserCtr.scan_twoway
        Updates:
            scandata - see LaserCtrl.scan and LaserCtrl.scan_twoway
        Notes:
            + This function runs forever. You'll want to run it in a thread.
            + Can be stopped during acquisition by self.stop=True
        """
        print("Starting scan loop with two_way={}".format(two_way))
        self.stop = False
        self.loop_scanning = True
        while not self.stop:
            if two_way:
                self.scan_twoway()
            else:
                self.scan()
            time.sleep(self.config["Scan"]["loop_delay"])
        time.sleep(0.2)
        self.laser.setWavelength(self.config["LaserSettings"]["wavelength"])
        self.stop = False
        self.loop_scanning = False

    def clear_data(self, kind="One way"):
        """Reinitilize dataset of a particular type
        Args:
            type - str {One way, Two way, Monitor}
        """
        arr0 = np.array(
            [
                0.0,
            ]
        )
        if kind == "One way":
            self.scandata = {
                "wav": arr0,
                "V": arr0,
            }
        elif kind == "Two way":
            self.scandata2way = {
                "wavstartstop": arr0,
                "Vstartstop": arr0,
                "wavstopstart": arr0,
                "Vstopstart": arr0,
            }
        elif kind == "Monitor":
            self.monitordata = [
                0,
            ]

    def save_scandata(self, save_config=True, overwrite_protect=True, saveplot=True):
        # save data, using dir and fname args in config['Save']
        savedir = self.config["Save"]["dir"]
        fname = self.config["Save"]["fname"]
        fpath = modeltools.overwrite_protection(
            savedir, fname, extension=".dat", overwrite_protect=overwrite_protect
        )
        header = "Data saved by LaserCtrl\n"
        if self.most_recent_scan == "One way":
            modeltools.save_data_dict(
                data_dict=self.scandata,
                fpath=fpath,
                keys=["wav", "V"],
                units=["nm", "V"],
                col_labels=["wavelength", "PD voltage"],
                header=header,
                config_dict=(self.config if save_config else None),
            )
        elif self.most_recent_scan == "Two way":
            modeltools.save_data_dict(
                data_dict=self.scandata2way,
                fpath=fpath,
                keys=["wavstartstop", "Vstartstop", "wavstopstart", "Vstopstart"],
                units=["nm", "V", "nm", "V"],
                col_labels=[
                    "wavelength start2stop",
                    "PD voltage start2stop",
                    "wavelength stop2start",
                    "PD voltage stop2start",
                ],
                header=header,
                config_dict=(self.config if save_config else None),
            )

        print("Data saved to: {}".format(fpath))
        if saveplot:
            if self.most_recent_scan == "One way":
                xs = self.scandata["wav"]
                ys = self.scandata["V"]
                legend = None
            elif self.most_recent_scan == "Two way":
                xs = [
                    self.scandata2way["wavstartstop"],
                    self.scandata2way["wavstopstart"],
                ]
                ys = [self.scandata2way["Vstartstop"], self.scandata2way["Vstopstart"]]
                legend = ["start>stop", "stop>start"]
            xlabel = "Wavelength (nm)"
            ylabel = "PD Voltage (V)"
            modeltools.save_data_fig(xs, ys, fpath, xlabel, ylabel, legend)

    def save_config(self, overwrite_protect=True):
        # save config,using dir and fname args in config['Save']
        fname = self.config["Save"]["fname"]
        savedir = self.config["Save"]["dir"]
        fpath = modeltools.overwrite_protection(
            savedir, fname, extension=".yml", overwrite_protect=overwrite_protect
        )
        modeltools.save_config(fpath, self.config)



# -*- coding: utf-8 -*-
"""
Created on Thu Feb  2 16:27:34 2023

@author: emmabat
"""

import re
import sys

import mariadb

sys.path.append(r"Q:\qnnpy")
import logging
from time import sleep

import serial

import qnnpy.functions.functions as qf
from qnnpy.instruments.cryocon34 import Cryocon34

"""
Python log levels:
    DEBUG
    INFO
    WARNING
    ERROR
    CRITICAL
"""
log_level = sys.argv[1]

try:
    logging.basicConfig(format="%(asctime)s %(message)s", level=log_level)
except Exception:
    logging.basicConfig(format="%(asctime)s %(message)s", level="ERROR")


def handler(self, signum, frame):
    """
    Safer exit handling (e.g. on ctrl+c)
    """
    conn.close()
    ser.close()
    exit(1)


def show_exception_and_exit(exc_type, exc_value, tb):
    import traceback

    traceback.print_exception(exc_type, exc_value, tb)
    input("Press key to exit.")
    sys.exit(-1)


def read_serial_pressure(ser):
    buffer = str(ser.read(64))

    logging.info(buffer)
    message = re.search("n[0-9], (.*)n", buffer)
    logging.info(message)

    try:
        pressure = float(message.group(0)[4:14])
        success = True
    except Exception:
        pressure = "NaN"
        success = False
    return pressure, success


sys.excepthook = show_exception_and_exit

COM_PORT = "COM3"

try:
    temp_reader = Cryocon34("GPIB0::5")
except Exception:
    logging.error("Failed to make VISA connection to Cryocon")
    exit
start_temp = 293

uname = "root"
pwd = "N0tqn&ngpwd"

try:
    conn = mariadb.connect(
        user=uname,
        password=pwd,
        host="qnn-nas-rle.mit.edu",
        port=3307,
        database="qnndb",
    )
except mariadb.Error as e:
    logging.error(f"Error connecting to MariaDB Platform: {e}")
    sys.exit(1)

# Get Cursor

with serial.Serial(COM_PORT, timeout=1) as ser, conn.cursor() as cur:
    pressure, success = read_serial_pressure(ser)
    failure_counter = 0
    while (not success) or failure_counter > 10:
        logging.info("failure counter: " + str(failure_counter))
        ser.write("COM,1\x0d\x0a".encode("ascii"))
        logging.debug("attempting to connect to serial")
        sleep(5)
        pressure, success = read_serial_pressure(ser)
        failure_counter += 1
    while True:
        sleep(30)
        logging.info("reading and writing")
        try:
            tempA = temp_reader.read_temp("A")
            tempB = temp_reader.read_temp("B")
            logging.info("read temperature success")
        except Exception:
            logging.error("failed to read temperature")
            tempA = 0
            tempB = 0
        try:
            pressure, success = read_serial_pressure(ser)
            if success:
                logging.info("read pressure success")
            else:
                pressure = 0
        except Exception:
            logging.error("failed to read pressure")
            pressure = 0

        try:
            qf.log_data_to_database(
                table_name="freespace_logging",
                channelA=tempA,
                channelB=tempB,
                pressure=pressure,
            )
            logging.info("write to database success")
        except Exception:
            logging.error("failed to write to database")
conn.close()



import serial


class SynthHD:
    allowed_chns = [-1, 0, 1]  # -1 means the last one

    def __init__(self, port="auto"):
        self.isinit = False
        self._cur_chn = 0
        self.on = [0, 0]
        self.freqs = [1000.0, 1000.0]
        self.powers = [0.0, 0.0]
        if port == "auto":
            # find if a SynthHD device is in the serial interface
            print("not implemented yet; please specify port manually")
            pass
        else:
            try:
                self.ser = serial.Serial(port)
                self.isinit = True
            except (OSError, serial.SerialException) as err:
                print("An error occurred when trying to connect to %s:\n" % port, err)
                print("Device not initialized")

    def __del__(self):
        if self.initialized():
            self.ser.close()

    def initialized(self):
        """Method to query if the port has been initialized"""
        return self.isinit

    ### channel related methods
    def _chn(self, chn):
        """return corrected channel number"""
        if chn in SynthHD.allowed_chns:
            return self._cur_chn if chn == -1 else chn
        else:
            print("Warning: Channel Invalid. Set to Default Channel " % self._cur_chn)

    def get_channel(self):
        """return the current channel"""
        return self._cur_chn

    def check_channel(self, chn):
        """check if the supplied"""
        return True if chn in SynthHD.allowed_chns else False

    def set_channel(self, chn):
        if self.check_channel(chn):
            self._cur_chn = self._chn(chn)
            self.ser.write(b"C%d" % chn)
        else:
            print("Error: Only Channel A(0) and B(1) are available")

    ### settings
    def _set_output(self, chn, val):
        self.set_channel(chn)
        # set power to high
        self.on[chn] = val
        self.ser.write(b"E%dr%d" % (val, val))

    def open(self, chn=-1):
        """-1 means the current"""
        if self.check_channel(chn):
            self._set_output(chn, 1)
        else:
            print("Channel Invalid: ", chn)

    def close(self, chn=-1):
        """-1 means the current"""
        if self.check_channel(chn):
            self._set_output(chn, 0)
        else:
            print("Channel Invalid: ", chn)

    def open_all(self):
        for chn in SynthHD.allowed_chns[1:]:
            self.open(chn)

    def close_all(self):
        for chn in SynthHD.allowed_chns[1:]:
            self.close(chn)

    def set_freq(self, freq):
        """in MHz"""
        if isinstance(freq, list):
            # validity check
            self.freqs = freq[: len(SynthHD.allowed_chns) - 1]
            for idx, chn in enumerate(SynthHD.allowed_chns[1:]):
                # print(chn, freq[idx])
                self.set_channel(chn)
                self.ser.write(b"f%.5f" % freq[idx])
        else:
            #        elif type(freq) in [float,int]:
            self.freqs[self._cur_chn] = freq  # freq[:len(SynthHD.allowed_chns)-1]
            self.set_channel(self._cur_chn)
            self.ser.write(b"f%.5f" % freq)

    #        else:
    #            print('Error: Frequency not valid. ', freq)

    def set_power(self, power):
        if isinstance(power, list):
            # validity check
            self.powers = power
            for idx, chn in enumerate(SynthHD.allowed_chns[1:]):
                # print(chn, power[idx])
                self.set_channel(chn)
                self.ser.write(b"W%.5f" % power[idx])

        else:  # type(power) in [float,int]:
            self.powers[self._cur_chn] = power  # freq[:len(SynthHD.allowed_chns)-1]
            self.set_channel(self._cur_chn)
            self.ser.write(b"W%.5f" % power)

    #        else:
    #            print('Error: Power not valid. ', power)

    def get_power(self):
        return self.powers


# def serial_ports():
#     """ Lists serial port names

#         :raises EnvironmentError:
#             On unsupported or unknown platforms
#         :returns:
#             A list of the serial ports available on the system
#     """
#     if sys.platform.startswith('win'):
#         ports = ['COM%s' % (i + 1) for i in range(256)]
#     elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
#         # this excludes your current terminal "/dev/tty"
#         ports = glob.glob('/dev/tty[A-Za-z]*')
#     elif sys.platform.startswith('darwin'):
#         ports = glob.glob('/dev/tty.*')
#     else:
#         raise EnvironmentError('Unsupported platform')

#     result = []
#     for port in ports:
#         try:
#             s = serial.Serial(port)
#             s.close()
#             result.append(port)
#         except (OSError, serial.SerialException):
#             pass

# serial_ports()
